export type BackgroundColor = "primary" | "primary-subtle" | "secondary" | "secondary-subtle" | "success" | "success-subtle" | "danger" | "danger-subtle" | "warning" | "warning-subtle" | "info" | "info-subtle" | "light" | "light-subtle" | "dark" | "dark-subtle" | "body" | "body-subtle" | "body-secondary" | "body-tertiary" | "black" | "white" | "transparent";
type BackgroundValue = BackgroundColor | "gradient";
interface BackgroundRecord {
    color?: BackgroundColor;
    gradient?: boolean;
}
export type Background = BackgroundValue | BackgroundRecord;
export type BackgroundOpacity = 100 | 75 | 50 | 25 | 10;
type BorderSideValue = "top" | "end" | "bottom" | "start";
interface BorderSideRecord extends Partial<Record<BorderSideValue, boolean | 0>> {
    all?: boolean | 0;
    top?: boolean | 0;
    end?: boolean | 0;
    bottom?: boolean | 0;
    start?: boolean | 0;
}
type BorderSide = BorderSideValue | BorderSideRecord;
type BorderColor = "primary" | "primary-subtle" | "secondary" | "secondary-subtle" | "success" | "success-subtle" | "danger" | "danger-subtle" | "warning" | "warning-subtle" | "info" | "info-subtle" | "light" | "light-subtle" | "dark" | "dark-subtle" | "white";
type BorderOpacity = 100 | 75 | 50 | 25 | 10;
type BorderWidth = 1 | 2 | 3 | 4 | 5;
type BorderValue = 0 | BorderSideValue | BorderColor | BorderWidth;
interface BorderRecord {
    border?: boolean | 0 | BorderSide;
    color?: BorderColor;
    opacity?: BorderOpacity;
    width?: BorderWidth;
}
export type Border = boolean | BorderValue | BorderRecord;
type RoundedSide = "top" | "end" | "bottom" | "start" | "circle" | "pill";
type RoundedSize = 0 | 1 | 2 | 3 | 4 | 5;
type RoundedValue = RoundedSide | RoundedSize;
interface RoundedRecord {
    rounded?: boolean | RoundedSide;
    size?: RoundedSize;
}
export type Rounded = boolean | RoundedValue | RoundedRecord;
export type Display = "none" | "inline" | "inline-block" | "block" | "grid" | "table" | "table-cell" | "table-row" | "flex" | "inline-flex";
export type FlexBreakpoint = "sm" | "md" | "lg" | "xl" | "xxl";
type FlexDirectionValue = "row" | "row-reverse" | "column" | "column-reverse";
type FlexWrapValue = "wrap" | "nowrap" | "wrap-reverse";
type FlexValue = FlexDirectionValue | "fill" | FlexWrapValue;
interface FlexRecord {
    direction?: FlexDirectionValue;
    fill?: boolean;
    wrap?: boolean | FlexWrapValue;
}
export type Flex = FlexValue | FlexRecord;
type FlexGrowValue = 0 | 1;
export type FlexGrow = boolean | FlexGrowValue;
type FlexShrinkValue = 0 | 1;
export type FlexShrink = boolean | FlexShrinkValue;
export type JustifyContent = "start" | "end" | "center" | "between" | "around" | "evenly";
export type AlignItems = "start" | "end" | "center" | "baseline" | "stretch";
export type AlignSelf = "start" | "end" | "center" | "between" | "around" | "evenly";
export type Order = 0 | 1 | 2 | 3 | 4 | 5 | "first" | "last";
export type AlignContent = "start" | "end" | "center" | "between" | "around" | "stretch";
export type Float = "start" | "end" | "none";
export type FloatBreakpoint = "sm" | "md" | "lg" | "xl" | "xxl";
export type UserSelect = "all" | "auto" | "none";
export type PointerEvent = "auto" | "none";
export type ObjectFit = "contain" | "cover" | "fill" | "scale" | "none";
export type Opacity = 100 | 75 | 50 | 25;
export type Overflow = "auto" | "hidden" | "visible" | "scroll";
export type Position = "static" | "relative" | "absolute" | "fixed" | "sticky";
type Arrange = 0 | 50 | 100;
export type Top = Arrange;
export type Bottom = Arrange;
export type Start = Arrange;
export type End = Arrange;
export type Translate = "middle" | "middle-x" | "middle-y";
export type Shadow = boolean | "none" | "sm" | "lg" | "inset";
type SizePercent = 25 | 50 | 75 | 100;
type Size = SizePercent | "auto";
export type Width = Size;
export type Height = Size;
type SizeLimited = 100;
export type MaxWidth = SizeLimited;
export type ViewportWidth = SizeLimited;
export type MinViewportWidth = SizeLimited;
export type MaxHeight = SizeLimited;
export type ViewportHeight = SizeLimited;
export type MinViewportHeight = SizeLimited;
type SpacingValue = 0 | 1 | 2 | 3 | 4 | 5;
export type SpacingSide = "t" | "b" | "s" | "e" | "x" | "y";
export type Padding = SpacingValue;
export type Margin = SpacingValue | "auto";
export type Gap = 0 | 1 | 2 | 3 | 4 | 5;
/** @deprecated */
type DeprecatedTextColor = "text-black-50" | "text-white-50" | "muted";
type TextColor = "primary" | "primary-emphasis" | "secondary" | "secondary-emphasis" | "success" | "success-emphasis" | "danger" | "danger-emphasis" | "warning" | "warning-emphasis" | "info" | "info-emphasis" | "light" | "light-emphasis" | "dark" | "dark-emphasis" | "body" | "body-emphasis" | "body-secondary" | "body-tertiary" | "black" | "white" | "reset" | DeprecatedTextColor;
type TextAlign = "start" | "center" | "end";
type TextTransform = "lowercase" | "uppercase" | "capitalize";
type TextValue = TextColor | TextAlign | "wrap" | "nowrap" | "break" | "truncate";
interface TextRecord {
    color?: TextColor;
    alignment?: TextAlign;
    wrap?: boolean;
    break?: boolean;
    transform?: TextTransform;
    truncate?: boolean;
}
export type Text = TextValue | TextRecord;
interface BreakpointTextRecord {
    alignment?: TextAlign;
}
export type BreakpointText = TextAlign | BreakpointTextRecord;
export type TextOpacity = 100 | 75 | 50 | 25;
export type TextDecoration = "underline" | "line-through" | "none";
export type FontSize = 1 | 2 | 3 | 4 | 5 | 6;
export type FontWeight = "bold" | "bolder" | "semibold" | "normal" | "light" | "lighter";
export type FontStyle = "italic" | "normal";
export type FontFamily = "monospace";
export type LineHeight = 1 | "sm" | "base" | "lg";
export type Align = "baseline" | "top" | "middle" | "bottom" | "text-bottom" | "text-top";
type VisibilityValue = "visible" | "invisible" | "visually-hidden" | "visually-hidden-focusable";
export type Visibility = boolean | VisibilityValue;
export type ZIndex = 0 | 1 | 2 | 3 | "n1";
export type Clearfix = boolean;
export type TextBackground = "primary" | "secondary" | "success" | "danger" | "warning" | "info" | "light" | "dark";
export type Link = "primary" | "secondary" | "success" | "danger" | "warning" | "info" | "light" | "dark";
export type Fixed = "top" | "bottom";
export type Sticky = "top" | "bottom";
export type StickyBreakpoint = "sm" | "md" | "lg" | "xl" | "xxl";
export type Ratio = "1x1" | "4x3" | "16x9" | "21x9";
export type VerticalStack = boolean;
export type HorizontalStack = boolean;
export type StretchedLink = boolean;
export type VerticalRule = boolean;
interface PeRecord {
    padding?: Padding;
    pointerEvent?: PointerEvent;
}
type Pe = Padding | PointerEvent | PeRecord;
export interface Utilities {
    align?: Align;
    alignContent?: AlignContent;
    alignItems?: AlignItems;
    alignSelf?: AlignSelf;
    bg?: Background;
    bgOpacity?: BackgroundOpacity;
    border?: Border;
    bottom?: Bottom;
    clearfix?: Clearfix;
    colGap?: Gap;
    display?: Display;
    end?: End;
    fixed?: Fixed;
    flex?: Flex;
    flexGrow?: FlexGrow;
    flexShrink?: FlexShrink;
    float?: Float;
    font?: FontFamily;
    fs?: FontSize;
    fst?: FontStyle;
    fw?: FontWeight;
    gap?: Gap;
    h?: Height;
    hstack?: HorizontalStack;
    justifyContent?: JustifyContent;
    lh?: LineHeight;
    link?: Link;
    m?: Margin;
    mb?: Margin;
    me?: Margin;
    mh?: MaxHeight;
    minVh?: MinViewportHeight;
    minVw?: MinViewportWidth;
    ms?: Margin;
    mt?: Margin;
    mw?: MaxWidth;
    mx?: Margin;
    my?: Margin;
    objectFit?: ObjectFit;
    opacity?: Opacity;
    order?: Order;
    overflow?: Overflow;
    overflowX?: Overflow;
    overflowY?: Overflow;
    p?: Padding;
    pb?: Padding;
    pe?: Pe;
    position?: Position;
    ps?: Padding;
    pt?: Padding;
    px?: Padding;
    py?: Padding;
    ratio?: Ratio;
    rounded?: Rounded;
    rowGap?: Gap;
    shadow?: Shadow;
    start?: Start;
    sticky?: Sticky;
    stretchedLink?: StretchedLink;
    text?: Text;
    textBg?: TextBackground;
    textDecoration?: TextDecoration;
    textOpacity?: TextOpacity;
    top?: Top;
    translate?: Translate;
    userSelect?: UserSelect;
    vh?: ViewportHeight;
    visibility?: Visibility;
    vr?: VerticalRule;
    vstack?: VerticalStack;
    vw?: ViewportWidth;
    w?: Width;
    z?: ZIndex;
    sm?: BreakpointUtilities;
    md?: BreakpointUtilities;
    lg?: BreakpointUtilities;
    xl?: BreakpointUtilities;
    xxl?: BreakpointUtilities;
    print?: Pick<BreakpointUtilities, "display">;
}
interface BreakpointUtilities {
    text?: BreakpointText;
    display?: Display;
    flex?: Flex;
    flexGrow?: FlexGrow;
    flexShrink?: FlexShrink;
    justifyContent?: JustifyContent;
    alignItems?: AlignItems;
    alignSelf?: AlignSelf;
    order?: Order;
    alignContent?: AlignContent;
    float?: Float;
    sticky?: Sticky;
}
export declare function utilityClassNames(props: Utilities | undefined): string[] | undefined;
export {};
