function isBackgroundColor(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeBgColor = value;
    return (maybeBgColor === "primary" ||
        maybeBgColor === "primary-subtle" ||
        maybeBgColor === "secondary" ||
        maybeBgColor === "secondary-subtle" ||
        maybeBgColor === "success" ||
        maybeBgColor === "success-subtle" ||
        maybeBgColor === "danger" ||
        maybeBgColor === "danger-subtle" ||
        maybeBgColor === "warning" ||
        maybeBgColor === "warning-subtle" ||
        maybeBgColor === "info" ||
        maybeBgColor === "info-subtle" ||
        maybeBgColor === "light" ||
        maybeBgColor === "light-subtle" ||
        maybeBgColor === "dark" ||
        maybeBgColor === "dark-subtle" ||
        maybeBgColor === "body" ||
        maybeBgColor === "body-subtle" ||
        maybeBgColor === "body-secondary" ||
        maybeBgColor === "body-tertiary" ||
        maybeBgColor === "black" ||
        maybeBgColor === "white" ||
        maybeBgColor === "transparent");
}
function isBackgroundValue(value) {
    return isBackgroundColor(value) || value === "gradient";
}
function pushBackgroundClassNames(classNames, bg) {
    if (isBackgroundValue(bg)) {
        classNames.push("bg-".concat(bg));
    }
    else if (bg && typeof bg === "object") {
        if (isBackgroundColor(bg.color))
            classNames.push("bg-".concat(bg.color));
        if (bg.gradient === true)
            classNames.push("bg-gradient");
    }
}
function isBackgroundOpacity(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeBgOpacity = value;
    return (maybeBgOpacity === 100 ||
        maybeBgOpacity === 75 ||
        maybeBgOpacity === 50 ||
        maybeBgOpacity === 25 ||
        maybeBgOpacity === 10);
}
function pushBackgroundOpacityClassNames(classNames, opacity) {
    if (isBackgroundOpacity(opacity)) {
        classNames.push("bg-opacity-".concat(opacity));
    }
}
function isBorderSideValue(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeBorderSide = value;
    return (maybeBorderSide === "top" ||
        maybeBorderSide === "end" ||
        maybeBorderSide === "bottom" ||
        maybeBorderSide === "start");
}
function isBorderColor(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeBorderColor = value;
    return (maybeBorderColor === "primary" ||
        maybeBorderColor === "primary-subtle" ||
        maybeBorderColor === "secondary" ||
        maybeBorderColor === "secondary-subtle" ||
        maybeBorderColor === "success" ||
        maybeBorderColor === "success-subtle" ||
        maybeBorderColor === "danger" ||
        maybeBorderColor === "danger-subtle" ||
        maybeBorderColor === "warning" ||
        maybeBorderColor === "warning-subtle" ||
        maybeBorderColor === "info" ||
        maybeBorderColor === "info-subtle" ||
        maybeBorderColor === "light" ||
        maybeBorderColor === "light-subtle" ||
        maybeBorderColor === "dark" ||
        maybeBorderColor === "dark-subtle" ||
        maybeBorderColor === "white");
}
function isBorderOpacity(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeBorderOpacity = value;
    return (maybeBorderOpacity === 100 ||
        maybeBorderOpacity === 75 ||
        maybeBorderOpacity === 50 ||
        maybeBorderOpacity === 25 ||
        maybeBorderOpacity === 10);
}
function isBorderWidth(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeBorderWidth = value;
    return (maybeBorderWidth === 1 ||
        maybeBorderWidth === 2 ||
        maybeBorderWidth === 3 ||
        maybeBorderWidth === 4 ||
        maybeBorderWidth === 5);
}
function isBorderValue(value) {
    return value === 0 || isBorderSideValue(value) || isBorderColor(value) || isBorderWidth(value);
}
function pushBorderClassNames(classNames, border) {
    if (border === true) {
        classNames.push("border");
    }
    else if (border === false || border === 0) {
        classNames.push("border-0");
    }
    else if (isBorderValue(border)) {
        classNames.push("border-".concat(border));
    }
    else if (border && typeof border === "object") {
        if (border.border === true) {
            classNames.push("border");
        }
        else if (border.border === false || border.border === 0) {
            classNames.push("border-0");
        }
        else if (isBorderSideValue(border.border)) {
            classNames.push("border-".concat(border.border));
        }
        else if (border.border && typeof border.border === "object") {
            if (border.border.all === true)
                classNames.push("border");
            if (border.border.all === false || border.border.all === 0)
                classNames.push("border-0");
            if (border.border.top === true)
                classNames.push("border-top");
            if (border.border.top === false || border.border.top === 0)
                classNames.push("border-top-0");
            if (border.border.bottom === true)
                classNames.push("border-bottom");
            if (border.border.bottom === false || border.border.bottom === 0)
                classNames.push("border-bottom-0");
            if (border.border.start === true)
                classNames.push("border-start");
            if (border.border.start === false || border.border.start === 0)
                classNames.push("border-start-0");
            if (border.border.end === true)
                classNames.push("border-end");
            if (border.border.end === false || border.border.end === 0)
                classNames.push("border-end-0");
        }
        if (isBorderColor(border.color))
            classNames.push("border-".concat(border.color));
        if (isBorderWidth(border.width))
            classNames.push("border-".concat(border.width));
        if (isBorderOpacity(border.opacity))
            classNames.push("border-opacity-".concat(border.opacity));
    }
}
function isRoundedSide(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeRoundedSide = value;
    return (maybeRoundedSide === "top" ||
        maybeRoundedSide === "end" ||
        maybeRoundedSide === "bottom" ||
        maybeRoundedSide === "start" ||
        maybeRoundedSide === "circle" ||
        maybeRoundedSide === "pill");
}
function isRoundedSize(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeRoundedSize = value;
    return (maybeRoundedSize === 0 ||
        maybeRoundedSize === 1 ||
        maybeRoundedSize === 2 ||
        maybeRoundedSize === 3 ||
        maybeRoundedSize === 4 ||
        maybeRoundedSize === 5);
}
function isRoundedValue(value) {
    return isRoundedSide(value) || isRoundedSize(value);
}
function pushRoundedClassNames(classNames, rounded) {
    if (rounded === true) {
        classNames.push("rounded");
    }
    else if (rounded === false) {
        classNames.push("rounded-0");
    }
    else if (isRoundedValue(rounded)) {
        classNames.push("rounded-".concat(rounded));
    }
    else if (rounded && typeof rounded === "object") {
        if (rounded.rounded === true) {
            classNames.push("rounded");
        }
        if (rounded.rounded === false) {
            classNames.push("rounded-0");
        }
        else if (isRoundedSize(rounded.size)) {
            classNames.push("rounded-".concat(rounded.size));
        }
        if (isRoundedSide(rounded.rounded)) {
            classNames.push("rounded-".concat(rounded.rounded));
        }
    }
}
function isDisplay(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeDisplay = value;
    return (maybeDisplay === "none" ||
        maybeDisplay === "inline" ||
        maybeDisplay === "inline-block" ||
        maybeDisplay === "block" ||
        maybeDisplay === "grid" ||
        maybeDisplay === "table" ||
        maybeDisplay === "table-cell" ||
        maybeDisplay === "table-row" ||
        maybeDisplay === "inline-flex");
}
function pushDisplayClassNames(classNames, display) {
    if (isDisplay(display)) {
        classNames.push("d-".concat(display));
    }
}
function pushBreakpointDisplayClassNames(classNames, breakpoint, display) {
    if (isDisplay(display)) {
        classNames.push("d-".concat(breakpoint, "-").concat(display));
    }
}
function isFlexDirectionValue(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFlexDirectionValue = value;
    return (maybeFlexDirectionValue === "row" ||
        maybeFlexDirectionValue === "row-reverse" ||
        maybeFlexDirectionValue === "column" ||
        maybeFlexDirectionValue === "column-reverse");
}
function isFlexWrapValue(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFlexWrap = value;
    return maybeFlexWrap === "wrap" || maybeFlexWrap === "nowrap" || maybeFlexWrap === "wrap-reverse";
}
function isFlexValue(value) {
    return isFlexDirectionValue(value) || value === "fill" || isFlexWrapValue(value);
}
function pushFlexClassNames(classNames, flex) {
    if (isFlexValue(flex)) {
        classNames.push("flex-".concat(flex));
        return;
    }
    if (flex && typeof flex === "object") {
        if (isFlexDirectionValue(flex.direction)) {
            classNames.push("flex-".concat(flex.direction));
        }
        if (flex.fill === true) {
            classNames.push("flex-fill");
        }
        if (flex.wrap === true) {
            classNames.push("flex-wrap");
        }
        else if (flex.wrap === false) {
            classNames.push("flex-nowrap");
        }
        else if (isFlexWrapValue(flex.wrap)) {
            classNames.push("flex-".concat(flex.wrap));
        }
    }
}
function pushBreakpointFlexClassNames(classNames, breakpoint, flex) {
    if (isFlexValue(flex)) {
        classNames.push("flex-".concat(breakpoint, "-").concat(flex));
        return;
    }
    if (flex && typeof flex === "object") {
        if (isFlexDirectionValue(flex.direction)) {
            classNames.push("flex-".concat(breakpoint, "-").concat(flex.direction));
        }
        if (flex.fill === true) {
            classNames.push("flex-".concat(breakpoint, "-fill"));
        }
        if (flex.wrap === true) {
            classNames.push("flex-".concat(breakpoint, "-wrap"));
        }
        else if (flex.wrap === false) {
            classNames.push("flex".concat(breakpoint, "-nowrap"));
        }
        else if (isFlexWrapValue(flex.wrap)) {
            classNames.push("flex-".concat(breakpoint, "-").concat(flex.wrap));
        }
    }
}
function isFlexGrowValue(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeFlexGrow = value;
    return maybeFlexGrow === 0 || maybeFlexGrow === 1;
}
function pushFlexGrowClassNames(classNames, grow) {
    if (grow === true) {
        classNames.push("flex-grow-1");
    }
    else if (grow === false) {
        classNames.push("flex-grow-0");
    }
    else if (isFlexGrowValue(grow)) {
        classNames.push("flex-grow-".concat(grow));
    }
}
function pushBreakpointFlexGrowClassNames(classNames, breakpoint, grow) {
    if (grow === true) {
        classNames.push("flex-grow-".concat(breakpoint, "-1"));
    }
    else if (grow === false) {
        classNames.push("flex-grow-".concat(breakpoint, "-0"));
    }
    else if (isFlexGrowValue(grow)) {
        classNames.push("flex-grow-".concat(breakpoint, "-").concat(grow));
    }
}
function isFlexShrinkValue(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeFlexShrinkValue = value;
    return maybeFlexShrinkValue === 0 || maybeFlexShrinkValue === 1;
}
function pushFlexShrinkClassNames(classNames, shrink) {
    if (shrink === true) {
        classNames.push("flex-shrink-1");
    }
    else if (shrink === false) {
        classNames.push("flex-shrink-0");
    }
    else if (isFlexShrinkValue(shrink)) {
        classNames.push("flex-shrink-".concat(shrink));
    }
}
function pushBreakpointFlexShrinkClassNames(classNames, breakpoint, shrink) {
    if (shrink === true) {
        classNames.push("flex-shrink-".concat(breakpoint, "-1"));
    }
    else if (shrink === false) {
        classNames.push("flex-shrink-".concat(breakpoint, "-0"));
    }
    else if (isFlexShrinkValue(shrink)) {
        classNames.push("flex-shrink-".concat(breakpoint, "-").concat(shrink));
    }
}
function isJustifyContent(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeJustifyContent = value;
    return (maybeJustifyContent === "start" ||
        maybeJustifyContent === "end" ||
        maybeJustifyContent === "center" ||
        maybeJustifyContent === "between" ||
        maybeJustifyContent === "around" ||
        maybeJustifyContent === "evenly");
}
function pushJustifyContentClassNames(classNames, justifyContent) {
    if (isJustifyContent(justifyContent)) {
        classNames.push("justify-content-".concat(justifyContent));
    }
}
function pushBreakpointJustifyContentClassNames(classNames, breakpoint, justifyContent) {
    if (isJustifyContent(justifyContent)) {
        classNames.push("justify-content-".concat(breakpoint, "-").concat(justifyContent));
    }
}
function isAlignItems(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeAlignItems = value;
    return (maybeAlignItems === "start" ||
        maybeAlignItems === "end" ||
        maybeAlignItems === "center" ||
        maybeAlignItems === "baseline" ||
        maybeAlignItems === "stretch");
}
function pushAlignItemsClassNames(classNames, alignItems) {
    if (isAlignItems(alignItems)) {
        classNames.push("align-items-".concat(alignItems));
    }
}
function pushBreakpointAlignItemsClassNames(classNames, breakpoint, alignItems) {
    if (isAlignItems(alignItems)) {
        classNames.push("align-items-".concat(breakpoint, "-").concat(alignItems));
    }
}
function isAlignSelf(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeAlignSelf = value;
    return (maybeAlignSelf === "start" ||
        maybeAlignSelf === "end" ||
        maybeAlignSelf === "center" ||
        maybeAlignSelf === "between" ||
        maybeAlignSelf === "around" ||
        maybeAlignSelf === "evenly");
}
function pushAlignSelfClassNames(classNames, alignSelf) {
    if (isAlignSelf(alignSelf)) {
        classNames.push("align-self-".concat(alignSelf));
    }
}
function pushBreakpointAlignSelfClassNames(classNames, breakpoint, alignSelf) {
    if (isAlignSelf(alignSelf)) {
        classNames.push("align-self-".concat(breakpoint, "-").concat(alignSelf));
    }
}
function isOrder(value) {
    if (typeof value !== "number" && typeof value !== "string") {
        return false;
    }
    var maybeOrder = value;
    return (maybeOrder === 0 ||
        maybeOrder === 1 ||
        maybeOrder === 2 ||
        maybeOrder === 3 ||
        maybeOrder === 4 ||
        maybeOrder === 5 ||
        maybeOrder === "first" ||
        maybeOrder === "last");
}
function pushOrderClassNames(classNames, order) {
    if (isOrder(order)) {
        classNames.push("order-".concat(order));
    }
}
function pushBreakpointOrderClassNames(classNames, breakpoint, order) {
    if (isOrder(order)) {
        classNames.push("order-".concat(breakpoint, "-").concat(order));
    }
}
function isAlignContent(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeAlignContent = value;
    return (maybeAlignContent === "start" ||
        maybeAlignContent === "end" ||
        maybeAlignContent === "center" ||
        maybeAlignContent === "between" ||
        maybeAlignContent === "around" ||
        maybeAlignContent === "stretch");
}
function pushAlignContentClassNames(classNames, alignContent) {
    if (isAlignContent(alignContent)) {
        classNames.push("align-content-".concat(alignContent));
    }
}
function pushBreakpointAlignContentClassNames(classNames, breakpoint, alignContent) {
    if (isAlignContent(alignContent)) {
        classNames.push("align-content-".concat(breakpoint, "-").concat(alignContent));
    }
}
function isFloat(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFloat = value;
    return maybeFloat === "start" || maybeFloat === "end" || maybeFloat === "none";
}
function pushFloatClassNames(classNames, float) {
    if (isFloat(float)) {
        classNames.push("float-".concat(float));
    }
}
function pushBreakpointFloatClassNames(classNames, breakpoint, float) {
    if (isFloat(float)) {
        classNames.push("float-".concat(breakpoint, "-").concat(float));
    }
}
function isUserSelect(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeUserSelect = value;
    return maybeUserSelect === "all" || maybeUserSelect === "auto" || maybeUserSelect === "none";
}
function pushUserSelectClassNames(classNames, userSelect) {
    if (isUserSelect(userSelect)) {
        classNames.push("user-select-".concat(userSelect));
    }
}
function isPointerEvent(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybePointerEvent = value;
    return maybePointerEvent === "auto" || maybePointerEvent === "none";
}
function pushPointerEventClassNames(classNames, pointerEvent) {
    if (isPointerEvent(pointerEvent)) {
        classNames.push("pe-".concat(pointerEvent));
    }
}
function isObjectFit(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeObjectFit = value;
    return (maybeObjectFit === "contain" ||
        maybeObjectFit === "cover" ||
        maybeObjectFit === "fill" ||
        maybeObjectFit === "scale" ||
        maybeObjectFit === "none");
}
function pushObjectFitClassNames(classNames, objectFit) {
    if (isObjectFit(objectFit)) {
        classNames.push("object-fit-".concat(objectFit));
    }
}
function isOpacity(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeOpacity = value;
    return maybeOpacity === 100 || maybeOpacity === 75 || maybeOpacity === 50 || maybeOpacity === 25;
}
function pushOpacityClassNames(classNames, opacity) {
    if (isOpacity(opacity)) {
        classNames.push("opacity-".concat(opacity));
    }
}
function isOverflow(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeOverflow = value;
    return (maybeOverflow === "auto" || maybeOverflow === "hidden" || maybeOverflow === "visible" || maybeOverflow === "scroll");
}
function pushOverflowClassNames(classNames, overflow) {
    if (isOverflow(overflow)) {
        classNames.push("overflow-".concat(overflow));
    }
}
function pushOverflowXClassNames(classNames, overflow) {
    if (isOverflow(overflow)) {
        classNames.push("overflow-x-".concat(overflow));
    }
}
function pushOverflowYClassNames(classNames, overflow) {
    if (isOverflow(overflow)) {
        classNames.push("overflow-y-".concat(overflow));
    }
}
function isPosition(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybePosition = value;
    return (maybePosition === "static" ||
        maybePosition === "relative" ||
        maybePosition === "absolute" ||
        maybePosition === "fixed" ||
        maybePosition === "sticky");
}
function pushPositionClassNames(classNames, position) {
    if (isPosition(position)) {
        classNames.push("position-".concat(position));
    }
}
function isArrange(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeArrange = value;
    return maybeArrange === 100 || maybeArrange === 50 || maybeArrange === 0;
}
function pushTopClassNames(classNames, top) {
    if (isArrange(top)) {
        classNames.push("top-".concat(top));
    }
}
function pushBottomClassNames(classNames, bottom) {
    if (isArrange(bottom)) {
        classNames.push("bottom-".concat(bottom));
    }
}
function pushStartClassNames(classNames, start) {
    if (isArrange(start)) {
        classNames.push("start-".concat(start));
    }
}
function pushEndClassNames(classNames, end) {
    if (isArrange(end)) {
        classNames.push("end-".concat(end));
    }
}
function isTranslate(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeTranslate = value;
    return maybeTranslate === "middle" || maybeTranslate === "middle-x" || maybeTranslate === "middle-y";
}
function pushTranslateClassNames(classNames, translate) {
    if (isTranslate(translate)) {
        classNames.push("translate-".concat(translate));
    }
}
function isShadow(value) {
    if (typeof value === "boolean") {
        return true;
    }
    if (typeof value !== "string") {
        return false;
    }
    var maybeShadow = value;
    return maybeShadow === "none" || maybeShadow === "sm" || maybeShadow === "lg" || maybeShadow === "inset";
}
function pushShadowClassNames(classNames, shadow) {
    if (shadow === true) {
        classNames.push("shadow");
    }
    else if (shadow === false) {
        classNames.push("shadow-none");
    }
    else if (isShadow(shadow)) {
        classNames.push("shadow-".concat(shadow));
    }
}
function isSizePercent(value) {
    if (typeof value !== "number") {
        return value;
    }
    var maybeSizePercent = value;
    return maybeSizePercent === 25 || maybeSizePercent === 50 || maybeSizePercent === 75 || maybeSizePercent === 100;
}
function isSize(value) {
    return isSizePercent(value) || value === "auto";
}
function pushWidthClassNames(classNames, width) {
    if (isSize(width)) {
        classNames.push("w-".concat(width));
    }
}
function pushHeightClassNames(classNames, height) {
    if (isSize(height)) {
        classNames.push("h-".concat(height));
    }
}
function isSizeLimited(value) {
    if (typeof value !== "number") {
        return value;
    }
    var maybeSize = value;
    return maybeSize === 100;
}
function pushMaxWidthClassNames(classNames, width) {
    if (isSizeLimited(width)) {
        classNames.push("mw-".concat(width));
    }
}
function pushViewportWidthClassNames(classNames, width) {
    if (isSizeLimited(width)) {
        classNames.push("vw-".concat(width));
    }
}
function pushMinViewportWidthClassNames(classNames, width) {
    if (isSizeLimited(width)) {
        classNames.push("min-vw-".concat(width));
    }
}
function pushMaxHeightClassNames(classNames, height) {
    if (isSizeLimited(height)) {
        classNames.push("mh-".concat(height));
    }
}
function pushViewportHeightClassNames(classNames, height) {
    if (isSizeLimited(height)) {
        classNames.push("vh-".concat(height));
    }
}
function pushMinViewportHeightClassNames(classNames, height) {
    if (isSizeLimited(height)) {
        classNames.push("min-vh-".concat(height));
    }
}
function isSpacingValue(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeSpacing = value;
    return (maybeSpacing === 0 ||
        maybeSpacing === 1 ||
        maybeSpacing === 2 ||
        maybeSpacing === 3 ||
        maybeSpacing === 4 ||
        maybeSpacing === 5);
}
function isSpacingSide(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeSpacingSide = value;
    return (maybeSpacingSide === "t" ||
        maybeSpacingSide === "b" ||
        maybeSpacingSide === "s" ||
        maybeSpacingSide === "e" ||
        maybeSpacingSide === "x" ||
        maybeSpacingSide === "y");
}
function pushPaddingClassNames(classNames, padding, side) {
    if (isSpacingValue(padding)) {
        if (isSpacingSide(side)) {
            classNames.push("p".concat(side, "-").concat(padding));
        }
        else {
            classNames.push("p-".concat(padding));
        }
    }
}
function isMargin(value) {
    return isSpacingValue(value) || value === "auto";
}
function pushMarginClassNames(classNames, margin, side) {
    if (isMargin(margin)) {
        if (isSpacingSide(side)) {
            classNames.push("m".concat(side, "-").concat(margin));
        }
        else {
            classNames.push("m-".concat(margin));
        }
    }
}
function isGap(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeGap = value;
    return maybeGap === 0 || maybeGap === 1 || maybeGap === 2 || maybeGap === 3 || maybeGap === 4 || maybeGap === 5;
}
function pushGapClassNames(classNames, gap) {
    if (isGap(gap)) {
        classNames.push("gap-".concat(gap));
    }
}
function pushRowGapClassNames(classNames, gap) {
    if (isGap(gap)) {
        classNames.push("row-gap-".concat(gap));
    }
}
function pushColumnGapClassNames(classNames, gap) {
    if (isGap(gap)) {
        classNames.push("col-gap-".concat(gap));
    }
}
function isTextColor(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeTextColor = value;
    return (maybeTextColor === "primary" ||
        maybeTextColor === "primary-emphasis" ||
        maybeTextColor === "secondary" ||
        maybeTextColor === "secondary-emphasis" ||
        maybeTextColor === "success" ||
        maybeTextColor === "success-emphasis" ||
        maybeTextColor === "danger" ||
        maybeTextColor === "danger-emphasis" ||
        maybeTextColor === "warning" ||
        maybeTextColor === "warning-emphasis" ||
        maybeTextColor === "info" ||
        maybeTextColor === "info-emphasis" ||
        maybeTextColor === "light" ||
        maybeTextColor === "light-emphasis" ||
        maybeTextColor === "dark" ||
        maybeTextColor === "dark-emphasis" ||
        maybeTextColor === "body" ||
        maybeTextColor === "muted" ||
        maybeTextColor === "body-emphasis" ||
        maybeTextColor === "body-secondary" ||
        maybeTextColor === "body-tertiary" ||
        maybeTextColor === "black" ||
        maybeTextColor === "white" ||
        maybeTextColor === "text-black-50" ||
        maybeTextColor === "text-white-50" ||
        maybeTextColor === "reset");
}
function isTextAlign(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeTextAlignment = value;
    return maybeTextAlignment === "start" || maybeTextAlignment === "center" || maybeTextAlignment === "end";
}
function isTextTransform(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeTextTransform = value;
    return (maybeTextTransform === "lowercase" || maybeTextTransform === "uppercase" || maybeTextTransform === "capitalize");
}
function isTextValue(value) {
    return (isTextColor(value) ||
        isTextAlign(value) ||
        value === "wrap" ||
        value === "nowrap" ||
        value === "break" ||
        value === "truncate");
}
function pushTextClassNames(classNames, text) {
    if (isTextValue(text)) {
        classNames.push("text-".concat(text));
        return;
    }
    if (text && typeof text === "object") {
        if (isTextColor(text.color))
            classNames.push("text-".concat(text.color));
        if (isTextAlign(text.alignment))
            classNames.push("text-".concat(text.alignment));
        if (text.wrap === true)
            classNames.push("text-wrap");
        if (text.wrap === false)
            classNames.push("text-nowrap");
        if (text["break"] === true)
            classNames.push("text-break");
        if (isTextTransform(text.transform))
            classNames.push("text-".concat(text.transform));
        if (text.truncate === true)
            classNames.push("text-truncate");
    }
}
function pushBreakpointTextClassNames(classNames, breakpoint, text) {
    if (isTextAlign(text)) {
        classNames.push("text-".concat(breakpoint, "-").concat(text));
        return;
    }
    if (text && typeof text === "object") {
        if (isTextAlign(text.alignment)) {
            classNames.push("text-".concat(breakpoint, "-").concat(text));
        }
    }
}
function isTextOpacity(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeTextOpacity = value;
    return maybeTextOpacity === 100 || maybeTextOpacity === 75 || maybeTextOpacity === 50 || maybeTextOpacity === 25;
}
function pushTextOpacityClassNames(classNames, opacity) {
    if (isTextOpacity(opacity)) {
        classNames.push("text-opacity-".concat(opacity));
    }
}
function isTextDecoration(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeTextDecoration = value;
    return (maybeTextDecoration === "underline" || maybeTextDecoration === "line-through" || maybeTextDecoration === "none");
}
function pushTextDecorationClassNames(classNames, decoration) {
    if (isTextDecoration(decoration)) {
        classNames.push("text-decoration-".concat(decoration));
    }
}
function isFontSize(value) {
    if (typeof value !== "number") {
        return false;
    }
    var maybeFontSize = value;
    return (maybeFontSize === 1 ||
        maybeFontSize === 2 ||
        maybeFontSize === 3 ||
        maybeFontSize === 4 ||
        maybeFontSize === 5 ||
        maybeFontSize === 6);
}
function pushFontSizeClassNames(classNames, fs) {
    if (isFontSize(fs)) {
        classNames.push("fs-".concat(fs));
    }
}
function isFontWeight(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFontWeight = value;
    return (maybeFontWeight === "bold" ||
        maybeFontWeight === "bolder" ||
        maybeFontWeight === "semibold" ||
        maybeFontWeight === "normal" ||
        maybeFontWeight === "light" ||
        maybeFontWeight === "lighter");
}
function pushFontWeightClassNames(classNames, fw) {
    if (isFontWeight(fw)) {
        classNames.push("fw-".concat(fw));
    }
}
function isFontStyle(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFontStyle = value;
    return maybeFontStyle === "italic" || maybeFontStyle === "normal";
}
function pushFontStyleClassNames(classNames, fst) {
    if (isFontStyle(fst)) {
        classNames.push("fst-".concat(fst));
    }
}
function isFontFamily(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFont = value;
    return maybeFont === "monospace";
}
function pushFontFamilyClassNames(classNames, font) {
    if (isFontFamily(font)) {
        classNames.push("font-".concat(font));
    }
}
function isLineHeight(value) {
    if (typeof value !== "number" && typeof value !== "string") {
        return false;
    }
    var maybeLineHeight = value;
    return maybeLineHeight === 1 || maybeLineHeight === "sm" || maybeLineHeight === "base" || maybeLineHeight === "lg";
}
function pushLineHeightClassNames(classNames, lh) {
    if (isLineHeight(lh)) {
        classNames.push("lh-".concat(lh));
    }
}
function isAlign(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeAlign = value;
    return (maybeAlign === "baseline" ||
        maybeAlign === "top" ||
        maybeAlign === "middle" ||
        maybeAlign === "bottom" ||
        maybeAlign === "text-bottom" ||
        maybeAlign === "text-top");
}
function pushAlignClassNames(classNames, align) {
    if (isAlign(align)) {
        classNames.push("align-".concat(align));
    }
}
function isVisibilityValue(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeVisibility = value;
    return (maybeVisibility === "visible" ||
        maybeVisibility === "invisible" ||
        maybeVisibility === "visually-hidden" ||
        maybeVisibility === "visually-hidden-focusable");
}
function pushVisibilityClassNames(classNames, visibility) {
    if (visibility === true) {
        classNames.push("visible");
    }
    else if (visibility === false) {
        classNames.push("invisible");
    }
    else if (isVisibilityValue(visibility)) {
        classNames.push(visibility);
    }
}
function isZIndex(value) {
    if (typeof value !== "number" && typeof value !== "string") {
        return false;
    }
    var maybeZ = value;
    return maybeZ === 0 || maybeZ === 1 || maybeZ === 2 || maybeZ === 3 || maybeZ === "n1";
}
function pushZIndexClassNames(classNames, z) {
    if (isZIndex(z)) {
        classNames.push("z-".concat(z));
    }
}
function pushClearfixClassNames(classNames, clearfix) {
    if (clearfix === true) {
        classNames.push("clearfix");
    }
}
function isTextBackground(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeTextBg = value;
    return (maybeTextBg === "primary" ||
        maybeTextBg === "secondary" ||
        maybeTextBg === "success" ||
        maybeTextBg === "danger" ||
        maybeTextBg === "warning" ||
        maybeTextBg === "info" ||
        maybeTextBg === "light" ||
        maybeTextBg === "dark");
}
function pushTextBackgroundClassNames(classNames, color) {
    if (isTextBackground(color)) {
        classNames.push("text-bg-".concat(color));
    }
}
function isLink(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeLinkColor = value;
    return (maybeLinkColor === "primary" ||
        maybeLinkColor === "secondary" ||
        maybeLinkColor === "success" ||
        maybeLinkColor === "danger" ||
        maybeLinkColor === "warning" ||
        maybeLinkColor === "info" ||
        maybeLinkColor === "light" ||
        maybeLinkColor === "dark");
}
function pushLinkClassNames(classNames, color) {
    if (isLink(color)) {
        classNames.push("link-".concat(color));
    }
}
function isFixed(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeFixed = value;
    return maybeFixed === "top" || maybeFixed === "bottom";
}
function pushFixedClassNames(classNames, fixed) {
    if (isFixed(fixed)) {
        classNames.push("fixed-".concat(fixed));
    }
}
function isSticky(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeSticky = value;
    return maybeSticky === "top" || maybeSticky === "bottom";
}
function pushStickyClassNames(classNames, sticky) {
    if (isSticky(sticky)) {
        classNames.push("sticky-".concat(sticky));
    }
}
function pushBreakpointStickyClassNames(classNames, breakpoint, sticky) {
    if (isSticky(sticky)) {
        classNames.push("sticky-".concat(breakpoint, "-").concat(sticky));
    }
}
function isRatio(value) {
    if (typeof value !== "string") {
        return false;
    }
    var maybeRatio = value;
    return maybeRatio === "1x1" || maybeRatio === "4x3" || maybeRatio === "16x9" || maybeRatio === "21x9";
}
function pushRatioClassNames(classNames, ratio) {
    if (isRatio(ratio)) {
        classNames.push("ratio");
        classNames.push("ratio-".concat(ratio));
    }
}
function pushVstackClassNames(classNames, vstack) {
    if (vstack === true) {
        classNames.push("vstack");
    }
}
function pushHstackClassNames(classNames, hstack) {
    if (hstack === true) {
        classNames.push("hstack");
    }
}
function pushStretchedLinkClassNames(classNames, stretchedLink) {
    if (stretchedLink === true) {
        classNames.push("stretched-link");
    }
}
function pushVrClassNames(classNames, vr) {
    if (vr === true) {
        classNames.push("vr");
    }
}
function pushPeClassNames(classNames, pe) {
    if (typeof pe === "string") {
        pushPointerEventClassNames(classNames, pe);
    }
    else if (typeof pe === "number") {
        pushPaddingClassNames(classNames, pe, "e");
    }
    else if (typeof pe === "object") {
        pushPaddingClassNames(classNames, pe.padding, "e");
        pushPointerEventClassNames(classNames, pe.pointerEvent);
    }
}
export function utilityClassNames(props) {
    if (!props) {
        return undefined;
    }
    var classNames = [];
    pushAlignClassNames(classNames, props.align);
    pushAlignContentClassNames(classNames, props.alignContent);
    pushAlignItemsClassNames(classNames, props.alignItems);
    pushAlignSelfClassNames(classNames, props.alignSelf);
    pushBackgroundClassNames(classNames, props.bg);
    pushBackgroundOpacityClassNames(classNames, props.bgOpacity);
    pushBorderClassNames(classNames, props.border);
    pushBottomClassNames(classNames, props.bottom);
    pushClearfixClassNames(classNames, props.clearfix);
    pushColumnGapClassNames(classNames, props.colGap);
    pushDisplayClassNames(classNames, props.display);
    pushEndClassNames(classNames, props.end);
    pushFixedClassNames(classNames, props.fixed);
    pushFlexClassNames(classNames, props.flex);
    pushFlexGrowClassNames(classNames, props.flexGrow);
    pushFlexShrinkClassNames(classNames, props.flexShrink);
    pushFloatClassNames(classNames, props.float);
    pushFontFamilyClassNames(classNames, props.font);
    pushFontSizeClassNames(classNames, props.fs);
    pushFontStyleClassNames(classNames, props.fst);
    pushFontWeightClassNames(classNames, props.fw);
    pushGapClassNames(classNames, props.gap);
    pushHeightClassNames(classNames, props.h);
    pushHstackClassNames(classNames, props.hstack);
    pushJustifyContentClassNames(classNames, props.justifyContent);
    pushLineHeightClassNames(classNames, props.lh);
    pushLinkClassNames(classNames, props.link);
    pushMarginClassNames(classNames, props.m, undefined);
    pushMarginClassNames(classNames, props.mb, "b");
    pushMarginClassNames(classNames, props.me, "e");
    pushMarginClassNames(classNames, props.ms, "s");
    pushMarginClassNames(classNames, props.mt, "t");
    pushMarginClassNames(classNames, props.mx, "x");
    pushMarginClassNames(classNames, props.my, "y");
    pushMaxHeightClassNames(classNames, props.mh);
    pushMaxWidthClassNames(classNames, props.mw);
    pushMinViewportHeightClassNames(classNames, props.minVh);
    pushMinViewportWidthClassNames(classNames, props.minVw);
    pushObjectFitClassNames(classNames, props.objectFit);
    pushOpacityClassNames(classNames, props.opacity);
    pushOrderClassNames(classNames, props.order);
    pushOverflowClassNames(classNames, props.overflow);
    pushOverflowXClassNames(classNames, props.overflowX);
    pushOverflowYClassNames(classNames, props.overflowY);
    pushPaddingClassNames(classNames, props.p, undefined);
    pushPaddingClassNames(classNames, props.pb, "b");
    pushPaddingClassNames(classNames, props.ps, "s");
    pushPaddingClassNames(classNames, props.pt, "t");
    pushPaddingClassNames(classNames, props.px, "x");
    pushPaddingClassNames(classNames, props.py, "y");
    pushPeClassNames(classNames, props.pe);
    pushPositionClassNames(classNames, props.position);
    pushRatioClassNames(classNames, props.ratio);
    pushRoundedClassNames(classNames, props.rounded);
    pushRowGapClassNames(classNames, props.rowGap);
    pushShadowClassNames(classNames, props.shadow);
    pushStartClassNames(classNames, props.start);
    pushStickyClassNames(classNames, props.sticky);
    pushStretchedLinkClassNames(classNames, props.stretchedLink);
    pushTextBackgroundClassNames(classNames, props.textBg);
    pushTextClassNames(classNames, props.text);
    pushTextDecorationClassNames(classNames, props.textDecoration);
    pushTextOpacityClassNames(classNames, props.textOpacity);
    pushTopClassNames(classNames, props.top);
    pushTranslateClassNames(classNames, props.translate);
    pushUserSelectClassNames(classNames, props.userSelect);
    pushViewportHeightClassNames(classNames, props.vh);
    pushViewportWidthClassNames(classNames, props.vw);
    pushVisibilityClassNames(classNames, props.visibility);
    pushVrClassNames(classNames, props.vr);
    pushVstackClassNames(classNames, props.vstack);
    pushWidthClassNames(classNames, props.w);
    pushZIndexClassNames(classNames, props.z);
    if (props.sm || props.md || props.lg || props.xl || props.xxl) {
        var rootProps = props;
        for (var _i = 0, _a = ["sm", "md", "lg", "xl", "xxl"]; _i < _a.length; _i++) {
            var breakpoint = _a[_i];
            var props_1 = rootProps[breakpoint];
            if (!props_1) {
                continue;
            }
            pushBreakpointAlignContentClassNames(classNames, breakpoint, props_1.alignContent);
            pushBreakpointAlignItemsClassNames(classNames, breakpoint, props_1.alignItems);
            pushBreakpointAlignSelfClassNames(classNames, breakpoint, props_1.alignSelf);
            pushBreakpointDisplayClassNames(classNames, breakpoint, props_1.display);
            pushBreakpointFlexClassNames(classNames, breakpoint, props_1.flex);
            pushBreakpointFlexGrowClassNames(classNames, breakpoint, props_1.flexGrow);
            pushBreakpointFlexShrinkClassNames(classNames, breakpoint, props_1.flexShrink);
            pushBreakpointFloatClassNames(classNames, breakpoint, props_1.float);
            pushBreakpointJustifyContentClassNames(classNames, breakpoint, props_1.justifyContent);
            pushBreakpointOrderClassNames(classNames, breakpoint, props_1.order);
            pushBreakpointStickyClassNames(classNames, breakpoint, props_1.sticky);
            pushBreakpointTextClassNames(classNames, breakpoint, props_1.text);
        }
    }
    if (props.print) {
        pushBreakpointDisplayClassNames(classNames, "print", props.display);
    }
    return classNames.length > 0 ? classNames : undefined;
}
