//#region Background
export type BackgroundColor =
  | "primary"
  | "primary-subtle"
  | "secondary"
  | "secondary-subtle"
  | "success"
  | "success-subtle"
  | "danger"
  | "danger-subtle"
  | "warning"
  | "warning-subtle"
  | "info"
  | "info-subtle"
  | "light"
  | "light-subtle"
  | "dark"
  | "dark-subtle"
  | "body"
  | "body-subtle"
  | "body-secondary"
  | "body-tertiary"
  | "black"
  | "white"
  | "transparent";

function isBackgroundColor(value: unknown): value is BackgroundColor {
  if (typeof value !== "string") {
    return false;
  }

  const maybeBgColor = value as BackgroundColor;
  return (
    maybeBgColor === "primary" ||
    maybeBgColor === "primary-subtle" ||
    maybeBgColor === "secondary" ||
    maybeBgColor === "secondary-subtle" ||
    maybeBgColor === "success" ||
    maybeBgColor === "success-subtle" ||
    maybeBgColor === "danger" ||
    maybeBgColor === "danger-subtle" ||
    maybeBgColor === "warning" ||
    maybeBgColor === "warning-subtle" ||
    maybeBgColor === "info" ||
    maybeBgColor === "info-subtle" ||
    maybeBgColor === "light" ||
    maybeBgColor === "light-subtle" ||
    maybeBgColor === "dark" ||
    maybeBgColor === "dark-subtle" ||
    maybeBgColor === "body" ||
    maybeBgColor === "body-subtle" ||
    maybeBgColor === "body-secondary" ||
    maybeBgColor === "body-tertiary" ||
    maybeBgColor === "black" ||
    maybeBgColor === "white" ||
    maybeBgColor === "transparent"
  );
}

type BackgroundValue = BackgroundColor | "gradient";

function isBackgroundValue(value: unknown): value is BackgroundValue {
  return isBackgroundColor(value) || value === "gradient";
}

interface BackgroundRecord {
  color?: BackgroundColor;
  gradient?: boolean;
}

export type Background = BackgroundValue | BackgroundRecord;

function pushBackgroundClassNames(classNames: string[], bg: Background | undefined): void {
  if (isBackgroundValue(bg)) {
    classNames.push(`bg-${bg}`);
  } else if (bg && typeof bg === "object") {
    if (isBackgroundColor(bg.color)) classNames.push(`bg-${bg.color}`);
    if (bg.gradient === true) classNames.push("bg-gradient");
  }
}

export type BackgroundOpacity = 100 | 75 | 50 | 25 | 10;

function isBackgroundOpacity(value: unknown): value is BackgroundOpacity {
  if (typeof value !== "number") {
    return false;
  }

  const maybeBgOpacity = value as BackgroundOpacity;
  return (
    maybeBgOpacity === 100 ||
    maybeBgOpacity === 75 ||
    maybeBgOpacity === 50 ||
    maybeBgOpacity === 25 ||
    maybeBgOpacity === 10
  );
}

function pushBackgroundOpacityClassNames(classNames: string[], opacity: BackgroundOpacity | undefined): void {
  if (isBackgroundOpacity(opacity)) {
    classNames.push(`bg-opacity-${opacity}`);
  }
}

//#endregion
//#region Borders
type BorderSideValue = "top" | "end" | "bottom" | "start";

function isBorderSideValue(value: unknown): value is BorderSideValue {
  if (typeof value !== "string") {
    return false;
  }

  const maybeBorderSide = value as BorderSideValue;
  return (
    maybeBorderSide === "top" ||
    maybeBorderSide === "end" ||
    maybeBorderSide === "bottom" ||
    maybeBorderSide === "start"
  );
}

interface BorderSideRecord extends Partial<Record<BorderSideValue, boolean | 0>> {
  all?: boolean | 0;
  top?: boolean | 0;
  end?: boolean | 0;
  bottom?: boolean | 0;
  start?: boolean | 0;
}

type BorderSide = BorderSideValue | BorderSideRecord;

type BorderColor =
  | "primary"
  | "primary-subtle"
  | "secondary"
  | "secondary-subtle"
  | "success"
  | "success-subtle"
  | "danger"
  | "danger-subtle"
  | "warning"
  | "warning-subtle"
  | "info"
  | "info-subtle"
  | "light"
  | "light-subtle"
  | "dark"
  | "dark-subtle"
  | "white";

function isBorderColor(value: unknown): value is BorderColor {
  if (typeof value !== "string") {
    return false;
  }

  const maybeBorderColor = value as BorderColor;
  return (
    maybeBorderColor === "primary" ||
    maybeBorderColor === "primary-subtle" ||
    maybeBorderColor === "secondary" ||
    maybeBorderColor === "secondary-subtle" ||
    maybeBorderColor === "success" ||
    maybeBorderColor === "success-subtle" ||
    maybeBorderColor === "danger" ||
    maybeBorderColor === "danger-subtle" ||
    maybeBorderColor === "warning" ||
    maybeBorderColor === "warning-subtle" ||
    maybeBorderColor === "info" ||
    maybeBorderColor === "info-subtle" ||
    maybeBorderColor === "light" ||
    maybeBorderColor === "light-subtle" ||
    maybeBorderColor === "dark" ||
    maybeBorderColor === "dark-subtle" ||
    maybeBorderColor === "white"
  );
}

type BorderOpacity = 100 | 75 | 50 | 25 | 10;

function isBorderOpacity(value: unknown): value is BorderOpacity {
  if (typeof value !== "number") {
    return false;
  }

  const maybeBorderOpacity = value as BorderOpacity;
  return (
    maybeBorderOpacity === 100 ||
    maybeBorderOpacity === 75 ||
    maybeBorderOpacity === 50 ||
    maybeBorderOpacity === 25 ||
    maybeBorderOpacity === 10
  );
}

type BorderWidth = 1 | 2 | 3 | 4 | 5;

function isBorderWidth(value: unknown): value is BorderWidth {
  if (typeof value !== "number") {
    return false;
  }

  const maybeBorderWidth = value as BorderWidth;
  return (
    maybeBorderWidth === 1 ||
    maybeBorderWidth === 2 ||
    maybeBorderWidth === 3 ||
    maybeBorderWidth === 4 ||
    maybeBorderWidth === 5
  );
}

type BorderValue = 0 | BorderSideValue | BorderColor | BorderWidth;

function isBorderValue(value: unknown): value is BorderValue {
  return value === 0 || isBorderSideValue(value) || isBorderColor(value) || isBorderWidth(value);
}

interface BorderRecord {
  border?: boolean | 0 | BorderSide;
  color?: BorderColor;
  opacity?: BorderOpacity;
  width?: BorderWidth;
}

export type Border = boolean | BorderValue | BorderRecord;

function pushBorderClassNames(classNames: string[], border: Border | undefined): void {
  if (border === true) {
    classNames.push("border");
  } else if (border === false || border === 0) {
    classNames.push("border-0");
  } else if (isBorderValue(border)) {
    classNames.push(`border-${border}`);
  } else if (border && typeof border === "object") {
    if (border.border === true) {
      classNames.push("border");
    } else if (border.border === false || border.border === 0) {
      classNames.push("border-0");
    } else if (isBorderSideValue(border.border)) {
      classNames.push(`border-${border.border}`);
    } else if (border.border && typeof border.border === "object") {
      if (border.border.all === true) classNames.push("border");
      if (border.border.all === false || border.border.all === 0) classNames.push("border-0");
      if (border.border.top === true) classNames.push("border-top");
      if (border.border.top === false || border.border.top === 0) classNames.push("border-top-0");
      if (border.border.bottom === true) classNames.push("border-bottom");
      if (border.border.bottom === false || border.border.bottom === 0) classNames.push("border-bottom-0");
      if (border.border.start === true) classNames.push("border-start");
      if (border.border.start === false || border.border.start === 0) classNames.push("border-start-0");
      if (border.border.end === true) classNames.push("border-end");
      if (border.border.end === false || border.border.end === 0) classNames.push("border-end-0");
    }
    if (isBorderColor(border.color)) classNames.push(`border-${border.color}`);
    if (isBorderWidth(border.width)) classNames.push(`border-${border.width}`);
    if (isBorderOpacity(border.opacity)) classNames.push(`border-opacity-${border.opacity}`);
  }
}

type RoundedSide = "top" | "end" | "bottom" | "start" | "circle" | "pill";

function isRoundedSide(value: unknown): value is RoundedSide {
  if (typeof value !== "string") {
    return false;
  }

  const maybeRoundedSide = value as RoundedSide;
  return (
    maybeRoundedSide === "top" ||
    maybeRoundedSide === "end" ||
    maybeRoundedSide === "bottom" ||
    maybeRoundedSide === "start" ||
    maybeRoundedSide === "circle" ||
    maybeRoundedSide === "pill"
  );
}

type RoundedSize = 0 | 1 | 2 | 3 | 4 | 5;

function isRoundedSize(value: unknown): value is RoundedSize {
  if (typeof value !== "number") {
    return false;
  }

  const maybeRoundedSize = value as RoundedSize;
  return (
    maybeRoundedSize === 0 ||
    maybeRoundedSize === 1 ||
    maybeRoundedSize === 2 ||
    maybeRoundedSize === 3 ||
    maybeRoundedSize === 4 ||
    maybeRoundedSize === 5
  );
}

type RoundedValue = RoundedSide | RoundedSize;

function isRoundedValue(value: unknown): value is RoundedValue {
  return isRoundedSide(value) || isRoundedSize(value);
}

interface RoundedRecord {
  rounded?: boolean | RoundedSide;
  size?: RoundedSize;
}

export type Rounded = boolean | RoundedValue | RoundedRecord;

function pushRoundedClassNames(classNames: string[], rounded: Rounded | undefined): void {
  if (rounded === true) {
    classNames.push("rounded");
  } else if (rounded === false) {
    classNames.push("rounded-0");
  } else if (isRoundedValue(rounded)) {
    classNames.push(`rounded-${rounded}`);
  } else if (rounded && typeof rounded === "object") {
    if (rounded.rounded === true) {
      classNames.push("rounded");
    }
    if (rounded.rounded === false) {
      classNames.push("rounded-0");
    } else if (isRoundedSize(rounded.size)) {
      classNames.push(`rounded-${rounded.size}`);
    }
    if (isRoundedSide(rounded.rounded)) {
      classNames.push(`rounded-${rounded.rounded}`);
    }
  }
}

//#endregion
//#region Display
export type Display =
  | "none"
  | "inline"
  | "inline-block"
  | "block"
  | "grid"
  | "table"
  | "table-cell"
  | "table-row"
  | "flex"
  | "inline-flex";

function isDisplay(value: unknown): value is Display {
  if (typeof value !== "string") {
    return false;
  }

  const maybeDisplay = value as Display;
  return (
    maybeDisplay === "none" ||
    maybeDisplay === "inline" ||
    maybeDisplay === "inline-block" ||
    maybeDisplay === "block" ||
    maybeDisplay === "grid" ||
    maybeDisplay === "table" ||
    maybeDisplay === "table-cell" ||
    maybeDisplay === "table-row" ||
    maybeDisplay === "inline-flex"
  );
}

function pushDisplayClassNames(classNames: string[], display: Display | undefined): void {
  if (isDisplay(display)) {
    classNames.push(`d-${display}`);
  }
}

function pushBreakpointDisplayClassNames(
  classNames: string[],
  breakpoint: "sm" | "md" | "lg" | "xl" | "xxl" | "print",
  display: Display | undefined
): void {
  if (isDisplay(display)) {
    classNames.push(`d-${breakpoint}-${display}`);
  }
}

//#endregion
//#region Flex
export type FlexBreakpoint = "sm" | "md" | "lg" | "xl" | "xxl";

type FlexDirectionValue = "row" | "row-reverse" | "column" | "column-reverse";

function isFlexDirectionValue(value: unknown): value is FlexDirectionValue {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFlexDirectionValue = value as FlexDirectionValue;
  return (
    maybeFlexDirectionValue === "row" ||
    maybeFlexDirectionValue === "row-reverse" ||
    maybeFlexDirectionValue === "column" ||
    maybeFlexDirectionValue === "column-reverse"
  );
}

type FlexWrapValue = "wrap" | "nowrap" | "wrap-reverse";

function isFlexWrapValue(value: unknown): value is FlexWrapValue {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFlexWrap = value as FlexWrapValue;
  return maybeFlexWrap === "wrap" || maybeFlexWrap === "nowrap" || maybeFlexWrap === "wrap-reverse";
}

type FlexValue = FlexDirectionValue | "fill" | FlexWrapValue;

function isFlexValue(value: unknown): value is FlexValue {
  return isFlexDirectionValue(value) || value === "fill" || isFlexWrapValue(value);
}

interface FlexRecord {
  direction?: FlexDirectionValue;
  fill?: boolean;
  wrap?: boolean | FlexWrapValue;
}

export type Flex = FlexValue | FlexRecord;

function pushFlexClassNames(classNames: string[], flex: Flex | undefined): void {
  if (isFlexValue(flex)) {
    classNames.push(`flex-${flex}`);
    return;
  }

  if (flex && typeof flex === "object") {
    if (isFlexDirectionValue(flex.direction)) {
      classNames.push(`flex-${flex.direction}`);
    }
    if (flex.fill === true) {
      classNames.push("flex-fill");
    }
    if (flex.wrap === true) {
      classNames.push("flex-wrap");
    } else if (flex.wrap === false) {
      classNames.push("flex-nowrap");
    } else if (isFlexWrapValue(flex.wrap)) {
      classNames.push(`flex-${flex.wrap}`);
    }
  }
}

function pushBreakpointFlexClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  flex: Flex | undefined
): void {
  if (isFlexValue(flex)) {
    classNames.push(`flex-${breakpoint}-${flex}`);
    return;
  }

  if (flex && typeof flex === "object") {
    if (isFlexDirectionValue(flex.direction)) {
      classNames.push(`flex-${breakpoint}-${flex.direction}`);
    }
    if (flex.fill === true) {
      classNames.push(`flex-${breakpoint}-fill`);
    }
    if (flex.wrap === true) {
      classNames.push(`flex-${breakpoint}-wrap`);
    } else if (flex.wrap === false) {
      classNames.push(`flex${breakpoint}-nowrap`);
    } else if (isFlexWrapValue(flex.wrap)) {
      classNames.push(`flex-${breakpoint}-${flex.wrap}`);
    }
  }
}

type FlexGrowValue = 0 | 1;

function isFlexGrowValue(value: unknown): value is FlexGrowValue {
  if (typeof value !== "number") {
    return false;
  }

  const maybeFlexGrow = value as FlexGrowValue;
  return maybeFlexGrow === 0 || maybeFlexGrow === 1;
}

export type FlexGrow = boolean | FlexGrowValue;

function pushFlexGrowClassNames(classNames: string[], grow: FlexGrow | undefined): void {
  if (grow === true) {
    classNames.push("flex-grow-1");
  } else if (grow === false) {
    classNames.push("flex-grow-0");
  } else if (isFlexGrowValue(grow)) {
    classNames.push(`flex-grow-${grow}`);
  }
}

function pushBreakpointFlexGrowClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  grow: FlexGrow | undefined
): void {
  if (grow === true) {
    classNames.push(`flex-grow-${breakpoint}-1`);
  } else if (grow === false) {
    classNames.push(`flex-grow-${breakpoint}-0`);
  } else if (isFlexGrowValue(grow)) {
    classNames.push(`flex-grow-${breakpoint}-${grow}`);
  }
}

type FlexShrinkValue = 0 | 1;

function isFlexShrinkValue(value: unknown): value is FlexShrinkValue {
  if (typeof value !== "number") {
    return false;
  }

  const maybeFlexShrinkValue = value as FlexShrinkValue;
  return maybeFlexShrinkValue === 0 || maybeFlexShrinkValue === 1;
}

export type FlexShrink = boolean | FlexShrinkValue;

function pushFlexShrinkClassNames(classNames: string[], shrink: FlexShrink | undefined): void {
  if (shrink === true) {
    classNames.push("flex-shrink-1");
  } else if (shrink === false) {
    classNames.push("flex-shrink-0");
  } else if (isFlexShrinkValue(shrink)) {
    classNames.push(`flex-shrink-${shrink}`);
  }
}

function pushBreakpointFlexShrinkClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  shrink: FlexShrink | undefined
): void {
  if (shrink === true) {
    classNames.push(`flex-shrink-${breakpoint}-1`);
  } else if (shrink === false) {
    classNames.push(`flex-shrink-${breakpoint}-0`);
  } else if (isFlexShrinkValue(shrink)) {
    classNames.push(`flex-shrink-${breakpoint}-${shrink}`);
  }
}

export type JustifyContent = "start" | "end" | "center" | "between" | "around" | "evenly";

function isJustifyContent(value: unknown): value is JustifyContent {
  if (typeof value !== "string") {
    return false;
  }

  const maybeJustifyContent = value as JustifyContent;
  return (
    maybeJustifyContent === "start" ||
    maybeJustifyContent === "end" ||
    maybeJustifyContent === "center" ||
    maybeJustifyContent === "between" ||
    maybeJustifyContent === "around" ||
    maybeJustifyContent === "evenly"
  );
}

function pushJustifyContentClassNames(classNames: string[], justifyContent: JustifyContent | undefined): void {
  if (isJustifyContent(justifyContent)) {
    classNames.push(`justify-content-${justifyContent}`);
  }
}

function pushBreakpointJustifyContentClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  justifyContent: JustifyContent | undefined
): void {
  if (isJustifyContent(justifyContent)) {
    classNames.push(`justify-content-${breakpoint}-${justifyContent}`);
  }
}

export type AlignItems = "start" | "end" | "center" | "baseline" | "stretch";

function isAlignItems(value: unknown): value is AlignItems {
  if (typeof value !== "string") {
    return false;
  }

  const maybeAlignItems = value as AlignItems;
  return (
    maybeAlignItems === "start" ||
    maybeAlignItems === "end" ||
    maybeAlignItems === "center" ||
    maybeAlignItems === "baseline" ||
    maybeAlignItems === "stretch"
  );
}

function pushAlignItemsClassNames(classNames: string[], alignItems: AlignItems | undefined): void {
  if (isAlignItems(alignItems)) {
    classNames.push(`align-items-${alignItems}`);
  }
}

function pushBreakpointAlignItemsClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  alignItems: AlignItems | undefined
): void {
  if (isAlignItems(alignItems)) {
    classNames.push(`align-items-${breakpoint}-${alignItems}`);
  }
}

export type AlignSelf = "start" | "end" | "center" | "between" | "around" | "evenly";

function isAlignSelf(value: unknown): value is AlignSelf {
  if (typeof value !== "string") {
    return false;
  }

  const maybeAlignSelf = value as AlignSelf;
  return (
    maybeAlignSelf === "start" ||
    maybeAlignSelf === "end" ||
    maybeAlignSelf === "center" ||
    maybeAlignSelf === "between" ||
    maybeAlignSelf === "around" ||
    maybeAlignSelf === "evenly"
  );
}

function pushAlignSelfClassNames(classNames: string[], alignSelf: AlignSelf | undefined): void {
  if (isAlignSelf(alignSelf)) {
    classNames.push(`align-self-${alignSelf}`);
  }
}

function pushBreakpointAlignSelfClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  alignSelf: AlignSelf | undefined
): void {
  if (isAlignSelf(alignSelf)) {
    classNames.push(`align-self-${breakpoint}-${alignSelf}`);
  }
}

export type Order = 0 | 1 | 2 | 3 | 4 | 5 | "first" | "last";

function isOrder(value: unknown): value is Order {
  if (typeof value !== "number" && typeof value !== "string") {
    return false;
  }

  const maybeOrder = value as Order;
  return (
    maybeOrder === 0 ||
    maybeOrder === 1 ||
    maybeOrder === 2 ||
    maybeOrder === 3 ||
    maybeOrder === 4 ||
    maybeOrder === 5 ||
    maybeOrder === "first" ||
    maybeOrder === "last"
  );
}

function pushOrderClassNames(classNames: string[], order: Order | undefined): void {
  if (isOrder(order)) {
    classNames.push(`order-${order}`);
  }
}

function pushBreakpointOrderClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  order: Order | undefined
): void {
  if (isOrder(order)) {
    classNames.push(`order-${breakpoint}-${order}`);
  }
}

export type AlignContent = "start" | "end" | "center" | "between" | "around" | "stretch";

function isAlignContent(value: unknown): value is AlignContent {
  if (typeof value !== "string") {
    return false;
  }

  const maybeAlignContent = value as AlignContent;
  return (
    maybeAlignContent === "start" ||
    maybeAlignContent === "end" ||
    maybeAlignContent === "center" ||
    maybeAlignContent === "between" ||
    maybeAlignContent === "around" ||
    maybeAlignContent === "stretch"
  );
}

function pushAlignContentClassNames(classNames: string[], alignContent: AlignContent | undefined): void {
  if (isAlignContent(alignContent)) {
    classNames.push(`align-content-${alignContent}`);
  }
}

function pushBreakpointAlignContentClassNames(
  classNames: string[],
  breakpoint: FlexBreakpoint,
  alignContent: AlignContent | undefined
): void {
  if (isAlignContent(alignContent)) {
    classNames.push(`align-content-${breakpoint}-${alignContent}`);
  }
}

//#endregion
//#region Float
export type Float = "start" | "end" | "none";

function isFloat(value: unknown): value is Float {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFloat = value as Float;
  return maybeFloat === "start" || maybeFloat === "end" || maybeFloat === "none";
}

export type FloatBreakpoint = "sm" | "md" | "lg" | "xl" | "xxl";

function pushFloatClassNames(classNames: string[], float: Float | undefined): void {
  if (isFloat(float)) {
    classNames.push(`float-${float}`);
  }
}

function pushBreakpointFloatClassNames(
  classNames: string[],
  breakpoint: FloatBreakpoint,
  float: Float | undefined
): void {
  if (isFloat(float)) {
    classNames.push(`float-${breakpoint}-${float}`);
  }
}

//#endregion
//#region Interactions
export type UserSelect = "all" | "auto" | "none";

function isUserSelect(value: unknown): value is UserSelect {
  if (typeof value !== "string") {
    return false;
  }

  const maybeUserSelect = value as UserSelect;
  return maybeUserSelect === "all" || maybeUserSelect === "auto" || maybeUserSelect === "none";
}

function pushUserSelectClassNames(classNames: string[], userSelect: UserSelect | undefined): void {
  if (isUserSelect(userSelect)) {
    classNames.push(`user-select-${userSelect}`);
  }
}

export type PointerEvent = "auto" | "none";

function isPointerEvent(value: unknown): value is PointerEvent {
  if (typeof value !== "string") {
    return false;
  }

  const maybePointerEvent = value as PointerEvent;
  return maybePointerEvent === "auto" || maybePointerEvent === "none";
}

function pushPointerEventClassNames(classNames: string[], pointerEvent: PointerEvent | undefined): void {
  if (isPointerEvent(pointerEvent)) {
    classNames.push(`pe-${pointerEvent}`);
  }
}

//#endregion
//#region ObjectFit
export type ObjectFit = "contain" | "cover" | "fill" | "scale" | "none";

function isObjectFit(value: unknown): value is ObjectFit {
  if (typeof value !== "string") {
    return false;
  }

  const maybeObjectFit = value as ObjectFit;
  return (
    maybeObjectFit === "contain" ||
    maybeObjectFit === "cover" ||
    maybeObjectFit === "fill" ||
    maybeObjectFit === "scale" ||
    maybeObjectFit === "none"
  );
}

function pushObjectFitClassNames(classNames: string[], objectFit: ObjectFit | undefined): void {
  if (isObjectFit(objectFit)) {
    classNames.push(`object-fit-${objectFit}`);
  }
}

//#endregion
//#region Opacity
export type Opacity = 100 | 75 | 50 | 25;

function isOpacity(value: unknown): value is Opacity {
  if (typeof value !== "number") {
    return false;
  }

  const maybeOpacity = value as Opacity;
  return maybeOpacity === 100 || maybeOpacity === 75 || maybeOpacity === 50 || maybeOpacity === 25;
}

function pushOpacityClassNames(classNames: string[], opacity: Opacity | undefined): void {
  if (isOpacity(opacity)) {
    classNames.push(`opacity-${opacity}`);
  }
}

//#endregion
//#region Overflow
export type Overflow = "auto" | "hidden" | "visible" | "scroll";

function isOverflow(value: unknown): value is Overflow {
  if (typeof value !== "string") {
    return false;
  }

  const maybeOverflow = value as Overflow;
  return (
    maybeOverflow === "auto" || maybeOverflow === "hidden" || maybeOverflow === "visible" || maybeOverflow === "scroll"
  );
}

function pushOverflowClassNames(classNames: string[], overflow: Overflow | undefined): void {
  if (isOverflow(overflow)) {
    classNames.push(`overflow-${overflow}`);
  }
}

function pushOverflowXClassNames(classNames: string[], overflow: Overflow | undefined): void {
  if (isOverflow(overflow)) {
    classNames.push(`overflow-x-${overflow}`);
  }
}

function pushOverflowYClassNames(classNames: string[], overflow: Overflow | undefined): void {
  if (isOverflow(overflow)) {
    classNames.push(`overflow-y-${overflow}`);
  }
}

//#endregion
//#region Position
export type Position = "static" | "relative" | "absolute" | "fixed" | "sticky";

function isPosition(value: unknown): value is Position {
  if (typeof value !== "string") {
    return false;
  }

  const maybePosition = value as Position;
  return (
    maybePosition === "static" ||
    maybePosition === "relative" ||
    maybePosition === "absolute" ||
    maybePosition === "fixed" ||
    maybePosition === "sticky"
  );
}

function pushPositionClassNames(classNames: string[], position: Position | undefined): void {
  if (isPosition(position)) {
    classNames.push(`position-${position}`);
  }
}

type Arrange = 0 | 50 | 100;

function isArrange(value: unknown): value is Arrange {
  if (typeof value !== "number") {
    return false;
  }

  const maybeArrange = value as Arrange;
  return maybeArrange === 100 || maybeArrange === 50 || maybeArrange === 0;
}

export type Top = Arrange;

function pushTopClassNames(classNames: string[], top: Top | undefined): void {
  if (isArrange(top)) {
    classNames.push(`top-${top}`);
  }
}

export type Bottom = Arrange;

function pushBottomClassNames(classNames: string[], bottom: Bottom | undefined): void {
  if (isArrange(bottom)) {
    classNames.push(`bottom-${bottom}`);
  }
}

export type Start = Arrange;

function pushStartClassNames(classNames: string[], start: Start | undefined): void {
  if (isArrange(start)) {
    classNames.push(`start-${start}`);
  }
}

export type End = Arrange;

function pushEndClassNames(classNames: string[], end: End | undefined): void {
  if (isArrange(end)) {
    classNames.push(`end-${end}`);
  }
}

export type Translate = "middle" | "middle-x" | "middle-y";

function isTranslate(value: unknown): value is Translate {
  if (typeof value !== "string") {
    return false;
  }

  const maybeTranslate = value as Translate;
  return maybeTranslate === "middle" || maybeTranslate === "middle-x" || maybeTranslate === "middle-y";
}

function pushTranslateClassNames(classNames: string[], translate: Translate | undefined): void {
  if (isTranslate(translate)) {
    classNames.push(`translate-${translate}`);
  }
}

//#endregion
//#region Shadows
export type Shadow = boolean | "none" | "sm" | "lg" | "inset";

function isShadow(value: unknown): value is Shadow {
  if (typeof value === "boolean") {
    return true;
  }

  if (typeof value !== "string") {
    return false;
  }

  const maybeShadow = value as Shadow;
  return maybeShadow === "none" || maybeShadow === "sm" || maybeShadow === "lg" || maybeShadow === "inset";
}

function pushShadowClassNames(classNames: string[], shadow: Shadow | undefined): void {
  if (shadow === true) {
    classNames.push("shadow");
  } else if (shadow === false) {
    classNames.push("shadow-none");
  } else if (isShadow(shadow)) {
    classNames.push(`shadow-${shadow}`);
  }
}

//#endregion
//#region Sizing
type SizePercent = 25 | 50 | 75 | 100;

function isSizePercent(value: any): value is SizePercent {
  if (typeof value !== "number") {
    return value;
  }

  const maybeSizePercent = value as SizePercent;
  return maybeSizePercent === 25 || maybeSizePercent === 50 || maybeSizePercent === 75 || maybeSizePercent === 100;
}

type Size = SizePercent | "auto";

function isSize(value: any): value is Size {
  return isSizePercent(value) || value === "auto";
}

export type Width = Size;

function pushWidthClassNames(classNames: string[], width?: Width): void {
  if (isSize(width)) {
    classNames.push(`w-${width}`);
  }
}

export type Height = Size;

function pushHeightClassNames(classNames: string[], height?: Height): void {
  if (isSize(height)) {
    classNames.push(`h-${height}`);
  }
}

type SizeLimited = 100;

function isSizeLimited(value: any): value is SizeLimited {
  if (typeof value !== "number") {
    return value;
  }

  const maybeSize = value as SizeLimited;
  return maybeSize === 100;
}

export type MaxWidth = SizeLimited;

function pushMaxWidthClassNames(classNames: string[], width?: MaxWidth): void {
  if (isSizeLimited(width)) {
    classNames.push(`mw-${width}`);
  }
}

export type ViewportWidth = SizeLimited;

function pushViewportWidthClassNames(classNames: string[], width?: ViewportWidth): void {
  if (isSizeLimited(width)) {
    classNames.push(`vw-${width}`);
  }
}

export type MinViewportWidth = SizeLimited;

function pushMinViewportWidthClassNames(classNames: string[], width?: MinViewportWidth): void {
  if (isSizeLimited(width)) {
    classNames.push(`min-vw-${width}`);
  }
}

export type MaxHeight = SizeLimited;

function pushMaxHeightClassNames(classNames: string[], height?: MaxHeight): void {
  if (isSizeLimited(height)) {
    classNames.push(`mh-${height}`);
  }
}

export type ViewportHeight = SizeLimited;

function pushViewportHeightClassNames(classNames: string[], height?: ViewportHeight): void {
  if (isSizeLimited(height)) {
    classNames.push(`vh-${height}`);
  }
}

export type MinViewportHeight = SizeLimited;

function pushMinViewportHeightClassNames(classNames: string[], height?: MinViewportHeight): void {
  if (isSizeLimited(height)) {
    classNames.push(`min-vh-${height}`);
  }
}

//#endregion
//#region Spacing
type SpacingValue = 0 | 1 | 2 | 3 | 4 | 5;

function isSpacingValue(value: unknown): value is SpacingValue {
  if (typeof value !== "number") {
    return false;
  }

  const maybeSpacing = value as SpacingValue;
  return (
    maybeSpacing === 0 ||
    maybeSpacing === 1 ||
    maybeSpacing === 2 ||
    maybeSpacing === 3 ||
    maybeSpacing === 4 ||
    maybeSpacing === 5
  );
}

export type SpacingSide = "t" | "b" | "s" | "e" | "x" | "y";

function isSpacingSide(value: unknown): value is SpacingSide {
  if (typeof value !== "string") {
    return false;
  }

  const maybeSpacingSide = value as SpacingSide;
  return (
    maybeSpacingSide === "t" ||
    maybeSpacingSide === "b" ||
    maybeSpacingSide === "s" ||
    maybeSpacingSide === "e" ||
    maybeSpacingSide === "x" ||
    maybeSpacingSide === "y"
  );
}

export type Padding = SpacingValue;

function pushPaddingClassNames(
  classNames: string[],
  padding: Padding | undefined,
  side: SpacingSide | undefined
): void {
  if (isSpacingValue(padding)) {
    if (isSpacingSide(side)) {
      classNames.push(`p${side}-${padding}`);
    } else {
      classNames.push(`p-${padding}`);
    }
  }
}

export type Margin = SpacingValue | "auto";

function isMargin(value: unknown): value is Margin {
  return isSpacingValue(value) || value === "auto";
}

function pushMarginClassNames(
  classNames: string[],
  margin: Margin | undefined,
  side: SpacingSide | undefined
): void {
  if (isMargin(margin)) {
    if (isSpacingSide(side)) {
      classNames.push(`m${side}-${margin}`);
    } else {
      classNames.push(`m-${margin}`);
    }
  }
}

export type Gap = 0 | 1 | 2 | 3 | 4 | 5;

function isGap(value: unknown): value is Gap {
  if (typeof value !== "number") {
    return false;
  }

  const maybeGap = value as Gap;
  return maybeGap === 0 || maybeGap === 1 || maybeGap === 2 || maybeGap === 3 || maybeGap === 4 || maybeGap === 5;
}

function pushGapClassNames(classNames: string[], gap?: Gap): void {
  if (isGap(gap)) {
    classNames.push(`gap-${gap}`);
  }
}

function pushRowGapClassNames(classNames: string[], gap?: Gap): void {
  if (isGap(gap)) {
    classNames.push(`row-gap-${gap}`);
  }
}

function pushColumnGapClassNames(classNames: string[], gap?: Gap): void {
  if (isGap(gap)) {
    classNames.push(`col-gap-${gap}`);
  }
}

//#endregion
//#region Text
/** @deprecated */
type DeprecatedTextColor = "text-black-50" | "text-white-50" | "muted";

type TextColor =
  | "primary"
  | "primary-emphasis"
  | "secondary"
  | "secondary-emphasis"
  | "success"
  | "success-emphasis"
  | "danger"
  | "danger-emphasis"
  | "warning"
  | "warning-emphasis"
  | "info"
  | "info-emphasis"
  | "light"
  | "light-emphasis"
  | "dark"
  | "dark-emphasis"
  | "body"
  | "body-emphasis"
  | "body-secondary"
  | "body-tertiary"
  | "black"
  | "white"
  | "reset"
  | DeprecatedTextColor;

function isTextColor(value: unknown): value is TextColor {
  if (typeof value !== "string") {
    return false;
  }

  const maybeTextColor = value as TextColor;
  return (
    maybeTextColor === "primary" ||
    maybeTextColor === "primary-emphasis" ||
    maybeTextColor === "secondary" ||
    maybeTextColor === "secondary-emphasis" ||
    maybeTextColor === "success" ||
    maybeTextColor === "success-emphasis" ||
    maybeTextColor === "danger" ||
    maybeTextColor === "danger-emphasis" ||
    maybeTextColor === "warning" ||
    maybeTextColor === "warning-emphasis" ||
    maybeTextColor === "info" ||
    maybeTextColor === "info-emphasis" ||
    maybeTextColor === "light" ||
    maybeTextColor === "light-emphasis" ||
    maybeTextColor === "dark" ||
    maybeTextColor === "dark-emphasis" ||
    maybeTextColor === "body" ||
    maybeTextColor === "muted" ||
    maybeTextColor === "body-emphasis" ||
    maybeTextColor === "body-secondary" ||
    maybeTextColor === "body-tertiary" ||
    maybeTextColor === "black" ||
    maybeTextColor === "white" ||
    maybeTextColor === "text-black-50" ||
    maybeTextColor === "text-white-50" ||
    maybeTextColor === "reset"
  );
}

type TextAlign = "start" | "center" | "end";

function isTextAlign(value: unknown): value is TextAlign {
  if (typeof value !== "string") {
    return false;
  }

  const maybeTextAlignment = value as TextAlign;
  return maybeTextAlignment === "start" || maybeTextAlignment === "center" || maybeTextAlignment === "end";
}

type TextTransform = "lowercase" | "uppercase" | "capitalize";

function isTextTransform(value: unknown): value is TextTransform {
  if (typeof value !== "string") {
    return false;
  }

  const maybeTextTransform = value as TextTransform;
  return (
    maybeTextTransform === "lowercase" || maybeTextTransform === "uppercase" || maybeTextTransform === "capitalize"
  );
}

type TextValue = TextColor | TextAlign | "wrap" | "nowrap" | "break" | "truncate";

function isTextValue(value: unknown): value is TextValue {
  return (
    isTextColor(value) ||
    isTextAlign(value) ||
    value === "wrap" ||
    value === "nowrap" ||
    value === "break" ||
    value === "truncate"
  );
}

interface TextRecord {
  color?: TextColor;
  alignment?: TextAlign;
  wrap?: boolean;
  break?: boolean;
  transform?: TextTransform;
  truncate?: boolean;
}

export type Text = TextValue | TextRecord;

function pushTextClassNames(classNames: string[], text: Text | undefined): void {
  if (isTextValue(text)) {
    classNames.push(`text-${text}`);
    return;
  }

  if (text && typeof text === "object") {
    if (isTextColor(text.color)) classNames.push(`text-${text.color}`);
    if (isTextAlign(text.alignment)) classNames.push(`text-${text.alignment}`);
    if (text.wrap === true) classNames.push("text-wrap");
    if (text.wrap === false) classNames.push("text-nowrap");
    if (text.break === true) classNames.push("text-break");
    if (isTextTransform(text.transform)) classNames.push(`text-${text.transform}`);
    if (text.truncate === true) classNames.push("text-truncate");
  }
}

interface BreakpointTextRecord {
  alignment?: TextAlign;
}

export type BreakpointText = TextAlign | BreakpointTextRecord;

function pushBreakpointTextClassNames(
  classNames: string[],
  breakpoint: "sm" | "md" | "lg" | "xl" | "xxl",
  text: BreakpointText | undefined
): void {
  if (isTextAlign(text)) {
    classNames.push(`text-${breakpoint}-${text}`);
    return;
  }

  if (text && typeof text === "object") {
    if (isTextAlign(text.alignment)) {
      classNames.push(`text-${breakpoint}-${text}`);
    }
  }
}

export type TextOpacity = 100 | 75 | 50 | 25;

function isTextOpacity(value: unknown): value is TextOpacity {
  if (typeof value !== "number") {
    return false;
  }

  const maybeTextOpacity = value as TextOpacity;
  return maybeTextOpacity === 100 || maybeTextOpacity === 75 || maybeTextOpacity === 50 || maybeTextOpacity === 25;
}

function pushTextOpacityClassNames(classNames: string[], opacity: TextOpacity | undefined): void {
  if (isTextOpacity(opacity)) {
    classNames.push(`text-opacity-${opacity}`);
  }
}

export type TextDecoration = "underline" | "line-through" | "none";

function isTextDecoration(value: unknown): value is TextDecoration {
  if (typeof value !== "string") {
    return false;
  }

  const maybeTextDecoration = value as TextDecoration;
  return (
    maybeTextDecoration === "underline" || maybeTextDecoration === "line-through" || maybeTextDecoration === "none"
  );
}

function pushTextDecorationClassNames(classNames: string[], decoration: TextDecoration | undefined): void {
  if (isTextDecoration(decoration)) {
    classNames.push(`text-decoration-${decoration}`);
  }
}

export type FontSize = 1 | 2 | 3 | 4 | 5 | 6;

function isFontSize(value: any): value is FontSize {
  if (typeof value !== "number") {
    return false;
  }

  const maybeFontSize = value as FontSize;
  return (
    maybeFontSize === 1 ||
    maybeFontSize === 2 ||
    maybeFontSize === 3 ||
    maybeFontSize === 4 ||
    maybeFontSize === 5 ||
    maybeFontSize === 6
  );
}

function pushFontSizeClassNames(classNames: string[], fs: FontSize | undefined): void {
  if (isFontSize(fs)) {
    classNames.push(`fs-${fs}`);
  }
}

export type FontWeight = "bold" | "bolder" | "semibold" | "normal" | "light" | "lighter";

function isFontWeight(value: unknown): value is FontWeight {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFontWeight = value as FontWeight;
  return (
    maybeFontWeight === "bold" ||
    maybeFontWeight === "bolder" ||
    maybeFontWeight === "semibold" ||
    maybeFontWeight === "normal" ||
    maybeFontWeight === "light" ||
    maybeFontWeight === "lighter"
  );
}

function pushFontWeightClassNames(classNames: string[], fw: FontWeight | undefined): void {
  if (isFontWeight(fw)) {
    classNames.push(`fw-${fw}`);
  }
}

export type FontStyle = "italic" | "normal";

function isFontStyle(value: unknown): value is FontStyle {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFontStyle = value as FontStyle;
  return maybeFontStyle === "italic" || maybeFontStyle === "normal";
}

function pushFontStyleClassNames(classNames: string[], fst: FontStyle | undefined): void {
  if (isFontStyle(fst)) {
    classNames.push(`fst-${fst}`);
  }
}

export type FontFamily = "monospace";

function isFontFamily(value: unknown): value is FontFamily {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFont = value as FontFamily;
  return maybeFont === "monospace";
}

function pushFontFamilyClassNames(classNames: string[], font: FontFamily | undefined): void {
  if (isFontFamily(font)) {
    classNames.push(`font-${font}`);
  }
}

export type LineHeight = 1 | "sm" | "base" | "lg";

function isLineHeight(value: unknown): value is LineHeight {
  if (typeof value !== "number" && typeof value !== "string") {
    return false;
  }

  const maybeLineHeight = value as LineHeight;
  return maybeLineHeight === 1 || maybeLineHeight === "sm" || maybeLineHeight === "base" || maybeLineHeight === "lg";
}

function pushLineHeightClassNames(classNames: string[], lh: LineHeight | undefined): void {
  if (isLineHeight(lh)) {
    classNames.push(`lh-${lh}`);
  }
}

//#endregion
//#region VerticalAlign
export type Align = "baseline" | "top" | "middle" | "bottom" | "text-bottom" | "text-top";

function isAlign(value: unknown): value is Align {
  if (typeof value !== "string") {
    return false;
  }

  const maybeAlign = value as Align;
  return (
    maybeAlign === "baseline" ||
    maybeAlign === "top" ||
    maybeAlign === "middle" ||
    maybeAlign === "bottom" ||
    maybeAlign === "text-bottom" ||
    maybeAlign === "text-top"
  );
}

function pushAlignClassNames(classNames: string[], align: Align | undefined): void {
  if (isAlign(align)) {
    classNames.push(`align-${align}`);
  }
}

//#endregion
//#region Visibility
type VisibilityValue = "visible" | "invisible" | "visually-hidden" | "visually-hidden-focusable";

function isVisibilityValue(value: unknown): value is VisibilityValue {
  if (typeof value !== "string") {
    return false;
  }

  const maybeVisibility = value as VisibilityValue;
  return (
    maybeVisibility === "visible" ||
    maybeVisibility === "invisible" ||
    maybeVisibility === "visually-hidden" ||
    maybeVisibility === "visually-hidden-focusable"
  );
}

export type Visibility = boolean | VisibilityValue;

function pushVisibilityClassNames(classNames: string[], visibility: Visibility | undefined): void {
  if (visibility === true) {
    classNames.push("visible");
  } else if (visibility === false) {
    classNames.push("invisible");
  } else if (isVisibilityValue(visibility)) {
    classNames.push(visibility);
  }
}

//#endregion
//#region ZIndex
export type ZIndex = 0 | 1 | 2 | 3 | "n1";

function isZIndex(value: unknown): value is ZIndex {
  if (typeof value !== "number" && typeof value !== "string") {
    return false;
  }

  const maybeZ = value as ZIndex;
  return maybeZ === 0 || maybeZ === 1 || maybeZ === 2 || maybeZ === 3 || maybeZ === "n1";
}

function pushZIndexClassNames(classNames: string[], z: ZIndex | undefined): void {
  if (isZIndex(z)) {
    classNames.push(`z-${z}`);
  }
}

//#endregion
//#region helpers/Clearfix
export type Clearfix = boolean;

function pushClearfixClassNames(classNames: string[], clearfix: Clearfix | undefined): void {
  if (clearfix === true) {
    classNames.push("clearfix");
  }
}

//#endregion
//#region helpers/ColorAndBackground
export type TextBackground = "primary" | "secondary" | "success" | "danger" | "warning" | "info" | "light" | "dark";

function isTextBackground(value: unknown): value is TextBackground {
  if (typeof value !== "string") {
    return false;
  }

  const maybeTextBg = value as TextBackground;
  return (
    maybeTextBg === "primary" ||
    maybeTextBg === "secondary" ||
    maybeTextBg === "success" ||
    maybeTextBg === "danger" ||
    maybeTextBg === "warning" ||
    maybeTextBg === "info" ||
    maybeTextBg === "light" ||
    maybeTextBg === "dark"
  );
}

function pushTextBackgroundClassNames(classNames: string[], color: TextBackground | undefined): void {
  if (isTextBackground(color)) {
    classNames.push(`text-bg-${color}`);
  }
}

//#endregion
//#region helpers/ColoredLinks
export type Link = "primary" | "secondary" | "success" | "danger" | "warning" | "info" | "light" | "dark";

function isLink(value: unknown): value is Link {
  if (typeof value !== "string") {
    return false;
  }

  const maybeLinkColor = value as Link;
  return (
    maybeLinkColor === "primary" ||
    maybeLinkColor === "secondary" ||
    maybeLinkColor === "success" ||
    maybeLinkColor === "danger" ||
    maybeLinkColor === "warning" ||
    maybeLinkColor === "info" ||
    maybeLinkColor === "light" ||
    maybeLinkColor === "dark"
  );
}

function pushLinkClassNames(classNames: string[], color: Link | undefined): void {
  if (isLink(color)) {
    classNames.push(`link-${color}`);
  }
}

//#endregion
//#region helpers/Position
export type Fixed = "top" | "bottom";

function isFixed(value: unknown): value is Fixed {
  if (typeof value !== "string") {
    return false;
  }

  const maybeFixed = value as Fixed;
  return maybeFixed === "top" || maybeFixed === "bottom";
}

function pushFixedClassNames(classNames: string[], fixed: Fixed | undefined): void {
  if (isFixed(fixed)) {
    classNames.push(`fixed-${fixed}`);
  }
}

export type Sticky = "top" | "bottom";

function isSticky(value: unknown): value is Sticky {
  if (typeof value !== "string") {
    return false;
  }

  const maybeSticky = value as Sticky;
  return maybeSticky === "top" || maybeSticky === "bottom";
}

function pushStickyClassNames(classNames: string[], sticky: Fixed | undefined): void {
  if (isSticky(sticky)) {
    classNames.push(`sticky-${sticky}`);
  }
}

export type StickyBreakpoint = "sm" | "md" | "lg" | "xl" | "xxl";

function pushBreakpointStickyClassNames(
  classNames: string[],
  breakpoint: StickyBreakpoint,
  sticky: Fixed | undefined
): void {
  if (isSticky(sticky)) {
    classNames.push(`sticky-${breakpoint}-${sticky}`);
  }
}

//#endregion
//#region helpers/Ratio
export type Ratio = "1x1" | "4x3" | "16x9" | "21x9";

function isRatio(value: unknown): value is Ratio {
  if (typeof value !== "string") {
    return false;
  }

  const maybeRatio = value as Ratio;
  return maybeRatio === "1x1" || maybeRatio === "4x3" || maybeRatio === "16x9" || maybeRatio === "21x9";
}

function pushRatioClassNames(classNames: string[], ratio: Ratio | undefined): void {
  if (isRatio(ratio)) {
    classNames.push("ratio");
    classNames.push(`ratio-${ratio}`);
  }
}

//#endregion
//#region helpers/Stacks
export type VerticalStack = boolean;

function pushVstackClassNames(classNames: string[], vstack: VerticalStack | undefined): void {
  if (vstack === true) {
    classNames.push("vstack");
  }
}

export type HorizontalStack = boolean;

function pushHstackClassNames(classNames: string[], hstack: HorizontalStack | undefined): void {
  if (hstack === true) {
    classNames.push("hstack");
  }
}

//#endregion
//#region helpers/StretchedLink
export type StretchedLink = boolean;

function pushStretchedLinkClassNames(classNames: string[], stretchedLink: StretchedLink | undefined): void {
  if (stretchedLink === true) {
    classNames.push("stretched-link");
  }
}

//#endregion
//#region helpers/VerticalRule
export type VerticalRule = boolean;

function pushVrClassNames(classNames: string[], vr: VerticalRule | undefined): void {
  if (vr === true) {
    classNames.push("vr");
  }
}

//#endregion
interface PeRecord {
  padding?: Padding;
  pointerEvent?: PointerEvent;
}

type Pe = Padding | PointerEvent | PeRecord;

function pushPeClassNames(classNames: string[], pe: Pe | undefined): void {
  if (typeof pe === "string") {
    pushPointerEventClassNames(classNames, pe);
  } else if (typeof pe === "number") {
    pushPaddingClassNames(classNames, pe, "e");
  } else if (typeof pe === "object") {
    pushPaddingClassNames(classNames, pe.padding, "e");
    pushPointerEventClassNames(classNames, pe.pointerEvent);
  }
}

export interface Utilities {
  align?: Align;
  alignContent?: AlignContent;
  alignItems?: AlignItems;
  alignSelf?: AlignSelf;
  bg?: Background;
  bgOpacity?: BackgroundOpacity;
  border?: Border;
  bottom?: Bottom;
  clearfix?: Clearfix;
  colGap?: Gap;
  display?: Display;
  end?: End;
  fixed?: Fixed;
  flex?: Flex;
  flexGrow?: FlexGrow;
  flexShrink?: FlexShrink;
  float?: Float;
  font?: FontFamily;
  fs?: FontSize;
  fst?: FontStyle;
  fw?: FontWeight;
  gap?: Gap;
  h?: Height;
  hstack?: HorizontalStack;
  justifyContent?: JustifyContent;
  lh?: LineHeight;
  link?: Link;
  m?: Margin;
  mb?: Margin;
  me?: Margin;
  mh?: MaxHeight;
  minVh?: MinViewportHeight;
  minVw?: MinViewportWidth;
  ms?: Margin;
  mt?: Margin;
  mw?: MaxWidth;
  mx?: Margin;
  my?: Margin;
  objectFit?: ObjectFit;
  opacity?: Opacity;
  order?: Order;
  overflow?: Overflow;
  overflowX?: Overflow;
  overflowY?: Overflow;
  p?: Padding;
  pb?: Padding;
  pe?: Pe;
  position?: Position;
  ps?: Padding;
  pt?: Padding;
  px?: Padding;
  py?: Padding;
  ratio?: Ratio;
  rounded?: Rounded;
  rowGap?: Gap;
  shadow?: Shadow;
  start?: Start;
  sticky?: Sticky;
  stretchedLink?: StretchedLink;
  text?: Text;
  textBg?: TextBackground;
  textDecoration?: TextDecoration;
  textOpacity?: TextOpacity;
  top?: Top;
  translate?: Translate;
  userSelect?: UserSelect;
  vh?: ViewportHeight;
  visibility?: Visibility;
  vr?: VerticalRule;
  vstack?: VerticalStack;
  vw?: ViewportWidth;
  w?: Width;
  z?: ZIndex;

  sm?: BreakpointUtilities;
  md?: BreakpointUtilities;
  lg?: BreakpointUtilities;
  xl?: BreakpointUtilities;
  xxl?: BreakpointUtilities;
  print?: Pick<BreakpointUtilities, "display">;
}

interface BreakpointUtilities {
  text?: BreakpointText;
  display?: Display;
  flex?: Flex;
  flexGrow?: FlexGrow;
  flexShrink?: FlexShrink;
  justifyContent?: JustifyContent;
  alignItems?: AlignItems;
  alignSelf?: AlignSelf;
  order?: Order;
  alignContent?: AlignContent;
  float?: Float;
  sticky?: Sticky;
}

export function utilityClassNames(props: Utilities | undefined): string[] | undefined {
  if (!props) {
    return undefined;
  }

  const classNames: string[] = [];

  pushAlignClassNames(classNames, props.align);
  pushAlignContentClassNames(classNames, props.alignContent);
  pushAlignItemsClassNames(classNames, props.alignItems);
  pushAlignSelfClassNames(classNames, props.alignSelf);
  pushBackgroundClassNames(classNames, props.bg);
  pushBackgroundOpacityClassNames(classNames, props.bgOpacity);
  pushBorderClassNames(classNames, props.border);
  pushBottomClassNames(classNames, props.bottom);
  pushClearfixClassNames(classNames, props.clearfix);
  pushColumnGapClassNames(classNames, props.colGap);
  pushDisplayClassNames(classNames, props.display);
  pushEndClassNames(classNames, props.end);
  pushFixedClassNames(classNames, props.fixed);
  pushFlexClassNames(classNames, props.flex);
  pushFlexGrowClassNames(classNames, props.flexGrow);
  pushFlexShrinkClassNames(classNames, props.flexShrink);
  pushFloatClassNames(classNames, props.float);
  pushFontFamilyClassNames(classNames, props.font);
  pushFontSizeClassNames(classNames, props.fs);
  pushFontStyleClassNames(classNames, props.fst);
  pushFontWeightClassNames(classNames, props.fw);
  pushGapClassNames(classNames, props.gap);
  pushHeightClassNames(classNames, props.h);
  pushHstackClassNames(classNames, props.hstack);
  pushJustifyContentClassNames(classNames, props.justifyContent);
  pushLineHeightClassNames(classNames, props.lh);
  pushLinkClassNames(classNames, props.link);
  pushMarginClassNames(classNames, props.m, undefined);
  pushMarginClassNames(classNames, props.mb, "b");
  pushMarginClassNames(classNames, props.me, "e");
  pushMarginClassNames(classNames, props.ms, "s");
  pushMarginClassNames(classNames, props.mt, "t");
  pushMarginClassNames(classNames, props.mx, "x");
  pushMarginClassNames(classNames, props.my, "y");
  pushMaxHeightClassNames(classNames, props.mh);
  pushMaxWidthClassNames(classNames, props.mw);
  pushMinViewportHeightClassNames(classNames, props.minVh);
  pushMinViewportWidthClassNames(classNames, props.minVw);
  pushObjectFitClassNames(classNames, props.objectFit);
  pushOpacityClassNames(classNames, props.opacity);
  pushOrderClassNames(classNames, props.order);
  pushOverflowClassNames(classNames, props.overflow);
  pushOverflowXClassNames(classNames, props.overflowX);
  pushOverflowYClassNames(classNames, props.overflowY);
  pushPaddingClassNames(classNames, props.p, undefined);
  pushPaddingClassNames(classNames, props.pb, "b");
  pushPaddingClassNames(classNames, props.ps, "s");
  pushPaddingClassNames(classNames, props.pt, "t");
  pushPaddingClassNames(classNames, props.px, "x");
  pushPaddingClassNames(classNames, props.py, "y");
  pushPeClassNames(classNames, props.pe);
  pushPositionClassNames(classNames, props.position);
  pushRatioClassNames(classNames, props.ratio);
  pushRoundedClassNames(classNames, props.rounded);
  pushRowGapClassNames(classNames, props.rowGap);
  pushShadowClassNames(classNames, props.shadow);
  pushStartClassNames(classNames, props.start);
  pushStickyClassNames(classNames, props.sticky);
  pushStretchedLinkClassNames(classNames, props.stretchedLink);
  pushTextBackgroundClassNames(classNames, props.textBg);
  pushTextClassNames(classNames, props.text);
  pushTextDecorationClassNames(classNames, props.textDecoration);
  pushTextOpacityClassNames(classNames, props.textOpacity);
  pushTopClassNames(classNames, props.top);
  pushTranslateClassNames(classNames, props.translate);
  pushUserSelectClassNames(classNames, props.userSelect);
  pushViewportHeightClassNames(classNames, props.vh);
  pushViewportWidthClassNames(classNames, props.vw);
  pushVisibilityClassNames(classNames, props.visibility);
  pushVrClassNames(classNames, props.vr);
  pushVstackClassNames(classNames, props.vstack);
  pushWidthClassNames(classNames, props.w);
  pushZIndexClassNames(classNames, props.z);

  if (props.sm || props.md || props.lg || props.xl || props.xxl) {
    const rootProps = props;
    for (const breakpoint of ["sm", "md", "lg", "xl", "xxl"] as const) {
      const props = rootProps[breakpoint];
      if (!props) {
        continue;
      }
      pushBreakpointAlignContentClassNames(classNames, breakpoint, props.alignContent);
      pushBreakpointAlignItemsClassNames(classNames, breakpoint, props.alignItems);
      pushBreakpointAlignSelfClassNames(classNames, breakpoint, props.alignSelf);
      pushBreakpointDisplayClassNames(classNames, breakpoint, props.display);
      pushBreakpointFlexClassNames(classNames, breakpoint, props.flex);
      pushBreakpointFlexGrowClassNames(classNames, breakpoint, props.flexGrow);
      pushBreakpointFlexShrinkClassNames(classNames, breakpoint, props.flexShrink);
      pushBreakpointFloatClassNames(classNames, breakpoint, props.float);
      pushBreakpointJustifyContentClassNames(classNames, breakpoint, props.justifyContent);
      pushBreakpointOrderClassNames(classNames, breakpoint, props.order);
      pushBreakpointStickyClassNames(classNames, breakpoint, props.sticky);
      pushBreakpointTextClassNames(classNames, breakpoint, props.text);
    }
  }
  if (props.print) {
    pushBreakpointDisplayClassNames(classNames, "print", props.display);
  }

  return classNames.length > 0 ? classNames : undefined;
}
